import os
import json
import logging
import datetime
import sys
from . import az_cis_utils as az_cis_utils

def check31():
    logging.info("Processing 31...")
    st31=""
    passvalue31 = 0
    totalvalue31 = 0
    passed31='<font color="red">Failed </font>'
    try:
        query31='az storage account list --query [*].[name,enableHttpsTrafficOnly]'
        json_cis=az_cis_utils.query_az(query31)
        if (len(json_cis)>0):
            #iteration through Storage Account
            for i in range(len(json_cis)):
                st31=st31+('Storage Account: <b>%s</b> status: <font color="blue"><b>%s</b></font><br>\n' % (json_cis[i][0],json_cis[i][1]))
                if (json_cis[i][1] is True):
                    passvalue31=passvalue31+1
                    passed31='<font color="green">Passed </font>'
                else:
                    passed31='<font color="red">Failed </font>'  
                totalvalue31 = totalvalue31+1
        else:
            totalvalue31 = 1
            passvalue31 = 1
            passed31='<font color="green">Passed </font>'
            st31="No Storage account found"
        score31=[st31,passvalue31,totalvalue31,passed31]
        return score31
    except Exception as e:
        logging.error("Exception in check31: %s %s" %(type(e), str(e.args)))
        st31="Failed to Query Storage Accounts"
        passed31='<font color="orange">UNKNOWN </font>'
        totalvalue31 = 1
        score31=[st31,passvalue31,totalvalue31,passed31]
        return score31
		
def check32():
    logging.info("Processing 32...")
    st32=""
    passvalue32 = 0
    totalvalue32 = 0
    score32=""
    passed32='<font color="red">Failed </font>'
    try:
        query32='az storage account list --query [*].[name,encryption.services.blob.enabled]'
        json_cis=az_cis_utils.query_az(query32)
        if (len(json_cis)>0):
            #iteration through Storage Account
            for i in range(len(json_cis)):
                st32=st32+('Storage Account: <b>%s</b> status: <font color="blue"><b>%s</b></font><br>\n' % (json_cis[i][0],json_cis[i][1]))
                if (json_cis[i][1] is True):
                    passvalue32=passvalue32+1
                    passed32='<font color="green">Passed </font>'
                else:
                    passed32='<font color="red">Failed </font>'  
                totalvalue32 = totalvalue32+1
        else:
            totalvalue32 = 1
            passvalue32 = 1
            passed32='<font color="green">Passed </font>'
            st32="No Storage account found"
        score32=[st32,passvalue32,totalvalue32,passed32]
        return score32
    except Exception as e:
        logging.error("Exception in check32: %s %s" %(type(e), str(e.args)))
        st32="Failed to Query Storage Accounts"
        passed32='<font color="orange">UNKNOWN </font>'
        totalvalue32 = 1
        score32=[st32,passvalue32,totalvalue32,passed32]
        return score32
		
def check33():
    logging.info("Processing 33... (Looking through logs for the past 90days")
    st33=""
    st331=""
    passvalue33 = 0
    totalvalue33 = 0
    score33=""
    passed33='<font color="green">Passed </font>'
    #query for the past 90 days. calculating today minus 90 days
    format = "%Y-%m-%dT%H:%M:%SZ"
    today = datetime.datetime.utcnow()
    #reduce time to speed up checks
    #pastdays = datetime.timedelta(days=10)
    pastdays = datetime.timedelta(days=90)
    todayminus90 = today - pastdays
    todayminus90 = todayminus90.strftime(format)
    try:
        query33='az storage account list --query [*].[resourceGroup,name]'
        json_cis=az_cis_utils.query_az(query33)
        if (len(json_cis)>0):
            #iteration through storage account Resource Group
            for i in range(len(json_cis)):
                RG = json_cis[i][0]
                StorageName = json_cis[i][1]
                found=0
                notfound=0
                logging.info("Checking storage %s within RG: %s", StorageName,RG)
                try:
                    query331='az monitor activity-log list --resource-group %s --start-time %s --query [*].[authorization.action,resourceId]' % (RG,todayminus90)
                    json_cis2=az_cis_utils.query_az(query331)
                    #iteration through event logs
                    if (len(json_cis2)>0):
                        for j in range(len(json_cis2)):
                            if ("Microsoft.Storage/storageAccounts/regenerateKey/action" in json_cis2[j][0]):
                                st331=('key regenerartion found for storage <b>%s</b> in resource group <b>%s</b><br>\n' % (StorageName,RG))
                                found=1
                                passvalue33=passvalue33+1
                                passed33='<font color="green">Passed </font>'
                            else:
                                notfound=1
                        if (found==0 and notfound==1):
                            st331=('No key regenerartion found for storage <b>%s</b> in resource group <b>%s</b><br>\n' % (StorageName,RG))
                            passed33='<font color="red">Failed </font>'  
                        totalvalue33 = totalvalue33+1
                        st33=st33+st331
                    else:
                        totalvalue33 = 1
                        st33=("No Event logs for the period in storage %s within RG: %s" % (StorageName,RG))
                except Exception as e:
                    logging.error("Exception in check33: %s %s" %(type(e), str(e.args)))
                    st33="Failed to Query Storage Accounts"
                    passed33='<font color="orange">UNKNOWN </font>'
                    totalvalue33 = 1
                    score33=[st33,passvalue33,totalvalue33,passed33]
                    return score33
        else:
            totalvalue33 = 1
            passvalue33 = 1
            passed33='<font color="green">Passed </font>'
            st33="No Storage account found"   
        score33=[st33,passvalue33,totalvalue33,passed33]
        return score33
    except Exception as e:
        logging.error("Exception in check33: %s %s" %(type(e), str(e.args)))
        st33="Failed to Query Storage Accounts"
        passed33='<font color="orange">UNKNOWN </font>'
        totalvalue33 = 1
        score33=[st33,passvalue33,totalvalue33,passed33]
        return score33


def check34():
    logging.info("Processing 34...")
    return "Check not available with azure CLI"

def check35():
    logging.info("Processing 35...")
    return "Check not available with azure CLI"

def check36():
    logging.info("Processing 36...")
    st36=""
    passvalue36 = 0
    totalvalue36 = 0
    score36=""
    passed36='<font color="red">Failed </font>'
    try:
        query36='az storage account list --query [*].[name,encryption.services.file.enabled]'
        json_cis=az_cis_utils.query_az(query36)
        if (len(json_cis)>0):
            #iteration through Storage Account
            for i in range(len(json_cis)):
                logging.info("Checking Storage Account: %s ", json_cis[i][0])
                st36=st36+('Storage Account: <b>%s</b> status: <font color="blue"><b>%s</b></font><br>\n' % (json_cis[i][0],json_cis[i][1]))
                if (json_cis[i][1] is True):
                    passvalue36=passvalue36+1
                    passed36='<font color="green">Passed </font>'
                else:
                    passed36='<font color="red">Failed </font>'  
                totalvalue36 = totalvalue36+1
        else:
            totalvalue36 = 1
            passvalue36 = 1
            passed36='<font color="green">Passed </font>'
            st36="No Storage account found" 
        score36=[st36,passvalue36,totalvalue36,passed36]
        return score36
    except Exception as e:
        logging.error("Exception in check36: %s %s" %(type(e), str(e.args)))
        st36="Failed to Query Storage Accounts"
        passed36='<font color="orange">UNKNOWN </font>'
        totalvalue36 = 1
        score36=[st36,passvalue36,totalvalue36,passed36]
        return score36

def check37():
    logging.info("Processing 37...")
    st37=""
    passvalue37 = 0
    totalvalue37 = 0
    score37=""
    passed37='<font color="green">Passed </font>'
    try:
        query37='az storage account list --query [*].[name,resourceGroup]'
        json_cis=az_cis_utils.query_az(query37)
        if (len(json_cis)>0):
            #iteration through Storage Account
            for i in range(len(json_cis)):
                SN=json_cis[i][0]
                RG=json_cis[i][1]
                logging.info("Checking storage %s within RG: %s", SN, RG)
                try:
                    query371=('az storage account keys list --account-name %s --resource-group %s --query [0].[value]' % (SN,RG))
                    json_cis2=az_cis_utils.query_az(query371)
                    KEY=json_cis2[0]
                    try:
                        query372=('az storage container list  --account-name %s --account-key %s --query [*].[properties.publicAccess,name]' % (SN,KEY))
                        json_cis3=az_cis_utils.query_az(query372)
                        #iteration through containers
                        if (len(json_cis3)>0):
                            for j in range(len(json_cis3)):
                                CT=json_cis3[j][1]
                                #print(json_cis3)
                                if (json_cis3[j][0] is None):
                                    st37=st37+('Private access for <b>%s</b> container in storage <b>%s</b><br>\n' % (CT,SN))
                                    passvalue37=passvalue37+1
                                else:
                                    st37=st37+('<font color="blue"><b>Public</b></font> access for <b>%s</b> container in storage <b>%s</b><br>\n' % (CT,SN))
                                    passed37='<font color="red">Failed </font>' 
                                totalvalue37 = totalvalue37+1
                        else:
                            totalvalue37 = 1
                            passvalue37 = 1
                            passed37='<font color="green">Passed </font>'
                            st37="No containers found" 
                    except Exception as e:
                        logging.error("Exception in check37: %s %s" %(type(e), str(e.args)))
                        st37="Failed iteration through containers"
                        passed37='<font color="orange">UNKNOWN </font>'
                        totalvalue37 = 1
                        score37=[st37,passvalue37,totalvalue37,passed37]
                        return score37
                except Exception as e:
                    logging.error("Exception in check37: %s %s" %(type(e), str(e.args)))
                    st37="Failed iteration through storage account"
                    passed37='<font color="orange">UNKNOWN </font>'
                    totalvalue37 = 1 
                    score37=[st37,passvalue37,totalvalue37,passed37]
                    return score37
        else:
            passvalue37 = 1
            totalvalue37 = 1
            passed37='<font color="green">Passed </font>'
            st37="No Storage account found" 
        score37=[st37,passvalue37,totalvalue37,passed37]
        return score37
    except Exception as e:
        logging.error("Exception in check37: %s %s" %(type(e), str(e.args)))
        st37="Failed to Query Storage Accounts"
        passed37='<font color="orange">UNKNOWN </font>'
        totalvalue37 = 1
        score37=[st37,passvalue37,totalvalue37,passed37]
        return score37



import sys
import subprocess
import os
import logging
import json

from . import check1 as check1
from . import check2 as check2
from . import check3 as check3
from . import check4 as check4
from . import check5 as check5
from . import check6 as check6
from . import check7 as check7
from . import gcp_cis_utils as gcp_cis_utils

def run_tests(args):
    gcp_cis_utils.set_encoding(args.encoding)
    gcp_cis_utils.set_expanded(args.expanded)
    if args.custom_ratings:
        if os.path.isfile(args.custom_ratings):
            with open(args.custom_ratings,"r") as fd:
                try:
                    temp_cr = json.load(fd)
                    custom_rating_dict = { }
                    for rating in temp_cr:
                        if rating not in ["1", "2", "3", "4", "5"]:
                            logging.error("Invalid rating [%s] specified in custom rating JSON file [%s]", rating, args.custom_ratings)
                            sys.exit(1)
                        tests = temp_cr[rating]
                        for test in tests:
                            custom_rating_dict[test] = rating
                    gcp_cis_utils.set_custom_ratings(custom_rating_dict)
                except ValueError as ve:
                    logging.error('Unable to load JSON file %s', args.custom_ratings)
                    logging.error(ve)
                    sys.exit(1)
        else:
            logging.error('Unable to access JSON file %s', args.custom_ratings)
            logging.error('Please check it exists and is accessible')
            sys.exit(1)

    config_issues = []
    p_not_found = []
    if args.projects:
        allprojs = gcp_cis_utils.get_all_projects()
        projs = args.projects.split(',')
        gcp_cis_utils._projects = []
        for p in projs:
            if p in allprojs:
                gcp_cis_utils._projects.append(p)
            else:
                p_not_found.append(p)

    if len(p_not_found) > 0:
        logging.error("Following project IDs were not found")
        logging.error(p_not_found)
        logging.error("Please provide correct project IDs and rerun")
        sys.exit(1)

    config_issues.extend(check1.run_checks())
    config_issues.extend(check2.run_checks())
    config_issues.extend(check3.run_checks())
    config_issues.extend(check4.run_checks())
    config_issues.extend(check5.run_checks())
    config_issues.extend(check6.run_checks())
    config_issues.extend(check7.run_checks())
    #print config_issues
    return config_issues

# run_tests()



import os
import json
import logging
from . import az_cis_utils as az_cis_utils

def check80():
    logging.info("Processing 81 and 82...")
    chk81=""
    chk82=""
    passvalue81 = 0
    totalvalue81 = 0
    score81=""
    passed81='<font color="green">Passed </font>'
    passvalue82 = 0
    totalvalue82 = 0
    score82=""
    passed82='<font color="green">Passed </font>'
    try:
        query80='az keyvault list --query [*].[name]'
        json_cis=az_cis_utils.query_az(query80)
        #iteration through keyvault
        if (len(json_cis)>0):
            try:
                for i in range(len(json_cis)):
                    query81=('az keyvault key list --vault-name %s --query [*].[kid,attributes.expires]' % json_cis[i][0])
                    json_cis2=az_cis_utils.query_az(query81)
                    #iteration through vault
                    for j in range(len(json_cis2)):
                        chk81=chk81+('Expiry date: <b>%s</b> for kid %s in key vault <b>%s</b><br>\n' % (json_cis2[j][1],json_cis2[j][0],json_cis[i][0]))
                        if (json_cis2[j][1]!=""):
                            passvalue81=passvalue81+1
                        totalvalue81 = totalvalue81+1
                    query82=('az keyvault secret list --vault-name %s --query [*].[id,attributes.expires]' % json_cis[i][0])
                    json_cis3=az_cis_utils.query_az(query82)
                    #iteration through vault
                    for j in range(len(json_cis3)):
                        chk82=chk82+('Expiry date: <b>%s</b> for id %s in key vault <b>%s</b><br>\n' % (json_cis3[j][1],json_cis3[j][0],json_cis[i][0]))
                        if (json_cis3[j][1]!=""):
                            passvalue82=passvalue82+1
                        totalvalue82 = totalvalue82+1
            except Exception as e:
                logging.error("Exception in check80: %s %s" %(type(e), str(e.args)))
                chk81="Failed to iterate through keyvault for VM"
                chk82="Failed to iterate through keyvault for VM"
                score81=[chk81,0,1,'<font color="orange">UNKNOWN </font>']
                score82=[chk82,0,1,'<font color="orange">UNKNOWN </font>']
                return [score81,score82] 
        else:
            chk81="No KeyVault Configured"
            chk82="No KeyVault Configured"
            totalvalue81=1
            totalvalue82=1
            passvalue81=1
            passvalue82=1
        score81=[chk81,passvalue81,totalvalue81,passed81]
        score82=[chk82,passvalue82,totalvalue82,passed82]
        return [score81,score82]  
    except Exception as e:
        logging.error("Exception in check80: %s %s" %(type(e), str(e.args)))
        chk81="Failed to query keyvault"
        chk82="Failed to query keyvault"
        score81=[chk81,0,1,'<font color="orange">UNKNOWN </font>']
        score82=[chk82,0,1,'<font color="orange">UNKNOWN </font>']
        return [score81,score82] 


def check83():
    logging.info("Processing 83...")
    chk83=""
    try:
        query83='az lock list --query [*].[level,name,resourceGroup]'
        json_cis=az_cis_utils.query_az(query83)
        if (len(json_cis)>0):
            #iteration through lock
            for i in range(len(json_cis)):
                chk83=chk83+('Lock status: <font color="blue"><b>%s</b></font> for lock <b><b>%s</b></b> in resource group <b>%s</b>s<br>\n' % (json_cis[i][0],json_cis[i][1],json_cis[i][2]))
        else:
            chk83="No Lock configured"
        return chk83
    except Exception as e:
        logging.error("Exception in check83: %s %s" %(type(e), str(e.args)))
        return "Failed to query lock"



import os
import sys
import logging
import json
import traceback
import ssl
import json
from pyVim.connect import SmartConnectNoSSL
from pyVmomi import vim

# Method that populates objects of type vimtype
def get_all_objs(content, vimtype):
    obj = {}
    container = content.viewManager.CreateContainerView(content.rootFolder, vimtype, True)
    for managed_object_ref in container.view:
        obj.update({managed_object_ref: managed_object_ref.name}),
    return obj

def discover(args):
    si = None
    try:
        pwd = os.environ.get('VCENTER_PASSWD')
        if pwd is None:
            if args.password is None:
                logging.error("vCenter password not found")
                return None
            pwd = args.password
        si = SmartConnectNoSSL(host=args.host, user=args.user, pwd=pwd)
    except:
         logging.error("Failed to connect to vCenter host "+args.host)
         return None
    if si == None:
         logging.error("Failed to connect to vCenter host "+args.host)
         return None

    content = si.content
    if content == None:
         logging.error("No information from vCenter host "+args.host)
         return None

    vmware_assets = []

    vcenter_asset = {}
    vcenter_asset['id'] = args.host
    vcenter_asset['name'] = args.host
    vcenter_asset['type'] = 'VMware vCenter'
    vcenter_asset['owner'] = args.handle
    plist = []
    plist.append(content.about.fullName)
    #plist.append(content.about.productLineId)
    #plist.append(content.about.name)
    vcenter_asset['products'] = plist
    vcenter_asset['tags'] = []
    vmware_assets.append(vcenter_asset)

    children = content.rootFolder.childEntity
    for child in children:  
        datacenter = child
        clusters = datacenter.hostFolder.childEntity
        for cluster in clusters:  
            hosts = cluster.host  
            for host in hosts:  
                esx_asset = {}
                summary = host.summary.config
                hostname = summary.name
                esx_asset['id'] = hostname
                esx_asset['name'] = hostname
                esx_asset['type'] = 'VMware ESXi'
                esx_asset['owner'] = args.handle
                plist = []
                plist.append(summary.product.fullName)
                #plist.append(summary.product.productLineId)
                #plist.append(summary.product.name)
                esx_asset['products'] = plist
                tags = []
                tags.append('vCenter DC: '+datacenter.name)
                tags.append('vCenter Cluster: '+cluster.name)
                tags.append('vCenter: '+args.host)
                esx_asset['tags'] = tags 
                vmware_assets.append(esx_asset)

    ajson = json.dumps(vmware_assets, indent=4)
    logging.debug(ajson)
    return vmware_assets

def get_inventory(args):
    return discover(args)



import os

def get_asset_id(args):
    asset_id = None
    if args.assetid == None or args.assetid.strip() == "":
        tokens = [args.handle.split('@')[0]]
        tokens.append(os.path.basename(os.path.normpath(args.input)).replace('.','-'))
        asset_id = "-".join(tokens)
    else:
        asset_id = args.assetid
    asset_id = asset_id.replace(' ','-')
    asset_id = asset_id.replace('/','-')
    asset_id = asset_id.replace(':','-')
    return asset_id

def convert_technology_products(technology_products, tags):
    for key in technology_products:
        tp_set = technology_products[key]
        tp_list = list(tp_set)
        technology_products[key] = tp_list
        tags.add(key)




import sys
import re
import os
import shutil
import stat
import subprocess
import logging
import json
import tempfile
import traceback

sast_plugin = "/usr/local/bin/semgrep"

def get_rating(severity):
    if severity == 'ERROR':
        return '4'
    if severity == 'WARNING':
        return '3'
    return '1'

def get_description(result):
    desc = result['extra']['message']
    if 'fix' in result['extra']:
        desc = desc + '\n' + 'Suggested fix: '+result['extra']['fix']
    if 'metadata' in result['extra']:
        if 'cwe' in result['extra']['metadata']:
            desc = desc + '\n' + 'CWE: '+result['extra']['metadata']['cwe']
        if 'owasp' in result['extra']['metadata']:
            owasp = result['extra']['metadata']['owasp']
            if isinstance(owasp, list):
                desc = desc + '\n' + 'OWASP: '+owasp[0]
            else:
                desc = desc + '\n' + 'OWASP: '+owasp
        if 'references' in result['extra']['metadata']:
            for ref in result['extra']['metadata']['references']:
                desc = desc + '\n' + ref
    return desc

def run_sast(args, path, base_path):
    findings = []
    if not os.path.isfile(sast_plugin) or not os.access(sast_plugin, os.X_OK):
        logging.warning('SAST plugin CLI - semgrep not found')
        return findings

    params = ' -q --json --config=p/r2c-security-audit ' + path
    
    cmdarr = [sast_plugin+ " " + params]
    sast_issues = None
    try:
        out = subprocess.check_output(cmdarr, shell=True)
        sast_issues = json.loads(out)
    except subprocess.CalledProcessError as e:
        logging.error("Error running SAST plugin CLI [semgrep]")
        logging.debug("[semgrep] command: %s", cmdarr[0])
        logging.debug("Output of [semgrep]: %s", e.output)
        return findings 
    logging.info("SAST plugin CLI [semgrep] checks completed")

    results = sast_issues['results']
    for r in results:
        finding = {}
        finding['issue_id'] = r['check_id']
        finding['rating'] = get_rating(r['extra']['severity'])
        finding['filename'] = r['path'].replace(base_path,'')
        if args.no_code:
            finding['code_snippet'] = ''
        else:
            finding['code_snippet'] = r['extra']['lines']
        finding['lineno_start'] = r['start']['line']
        finding['lineno_end'] = r['end']['line']
        finding['description'] = get_description(r)
        finding['type'] = 'SAST'
        finding['cwe'] = ''
        finding['owasp'] = ''
        if 'metadata' in r['extra']:
            if 'cwe' in r['extra']['metadata']:
                finding['cwe'] = r['extra']['metadata']['cwe']
            if 'owasp' in r['extra']['metadata']:
                owasp = r['extra']['metadata']['owasp']
                if isinstance(owasp, list):
                    finding['owasp'] = owasp[0]
                else:
                    finding['owasp'] = owasp
        findings.append(finding)

    return findings



import sys
import os
import subprocess
import logging
import re
import json

from . import utils

def run_k8s_cis(args, ctype):
    k8s_cis_path = os.path.dirname(os.path.realpath(__file__)) + '/kubernetes-cis-benchmark/'
    if args.target == None or (args.target != 'master' and args.target != 'worker'):
        logging.error("No target [master | worker] specified or invalid target")
        sys.exit(1)

    if args.assetid.strip() == "":
        logging.error("[assetid] cannot be empty")
        sys.exit(1)

    custom_rating_dict = None
    if args.custom_ratings:
        if os.path.isfile(args.custom_ratings):
            with open(args.custom_ratings,"r") as fd:
                try:
                    temp_cr = json.load(fd)
                    custom_rating_dict = { }
                    for rating in temp_cr:
                        if rating not in ["1", "2", "3", "4", "5"]:
                            logging.error("Invalid rating [%s] specified in custom rating JSON file [%s]", rating, args.custom_ratings)
                            sys.exit(1)
                        tests = temp_cr[rating]
                        for test in tests:
                            custom_rating_dict[test] = rating
                except ValueError as ve:
                    logging.error('Unable to load JSON file %s', args.custom_ratings)
                    logging.error(ve)
                    sys.exit(1)
        else:
            logging.error('Unable to access JSON file %s', args.custom_ratings)
            logging.error('Please check it exists and is accessible')
            sys.exit(1)

    asset = {}
    asset['id'] = args.assetid
    asset['name'] = args.assetname if args.assetname and args.assetname.strip() != "" else asset['id']
    if ctype == 'k8s':
        asset['type'] = 'Kubernetes'
        asset['tags'] = ['Kubernetes', 'CIS']
    else:
        asset['type'] = 'GKE'
        asset['tags'] = ['GKE', 'CIS']
    asset['owner'] = args.handle
    asset['products'] = []
    args.no_scan = True


    k8s_cmd = k8s_cis_path
    if args.target == 'master':
        if ctype == 'k8s': 
            k8s_cmd = '/bin/bash ./master.sh 1.6.0'
        else:
            k8s_cmd = '/bin/bash ./master.sh gke'
    else:
        if ctype == 'k8s': 
            k8s_cmd = '/bin/bash ./worker.sh 1.6.0'
        else:
            k8s_cmd = '/bin/bash ./worker.sh gke'
    logging.info('Running CIS benchmark: ')
    try:
        os.chdir(os.path.dirname(k8s_cis_path))
        out = subprocess.check_output([k8s_cmd+" 2>/dev/null"], shell=True)
        out = out.decode(args.encoding)
        ansi_escape = re.compile(r'(?:\x1B[@-_]|[\x80-\x9F])[0-?]*[ -/]*[@-~]')
        out = ansi_escape.sub('', out)
    except subprocess.CalledProcessError:
        logging.error("Error running CIS script")
        return None
    logging.info("CIS run completed")

    findings = []
    details = ''
    for l in out.splitlines():
        if not l.startswith('[WARN]'):
            continue
        issue = {}
        spa = l.split()
        check_id = spa[1].strip()
        if ctype == 'gke':
            check_id = l.split('Scored]')[1].strip().split()[0]
        if ctype == 'k8s':
            issue['twc_id'] = 'kubernetes-cis-check-'+check_id
        else:
            issue['twc_id'] = 'gke-cis-check-'+check_id
        issue['asset_id'] = args.assetid 
        issue['twc_title'] = check_id + ' ' + l.split(check_id)[1].strip()
        if custom_rating_dict:
            rating = custom_rating_dict.get(check_id)
            if rating == None:
                rating = '4'
        else:
            rating = '4'
        issue['rating'] = rating 
        if ctype == 'k8s':
            issue['type'] = 'Kubernetes CIS'
        else:
            issue['type'] = 'GKE CIS'
        issue['object_id'] = ''
        issue['object_meta'] = ''
        details = issue['twc_title'].replace(check_id+' - ','')
        issue['details'] = details
        findings.append(issue)

    asset['config_issues'] = findings
    #print(json.dumps(asset, indent=4))
    return [ asset ]

def get_inventory(args, ctype):
    return run_k8s_cis(args, ctype)



import sys
import re
import os
import shutil
import stat
import subprocess
import logging
import json
import tempfile
import hashlib
import traceback

def on_rm_error( func, path, exc_info):
    os.chmod( path, stat.S_IWRITE )
    os.unlink( path )

def tw_open(in_file, in_encoding):
    if sys.version_info[0] < 3:
        f = open(in_file)
    else:
        f = open(in_file, encoding=in_encoding)
    return f

def get_rating(severity):
    if severity == 'informational':
        return '1'
    if severity == 'low':
        return '3'
    if severity == 'medium':
        return '4'
    if severity == 'high':
        return '5'

def parse_arachni(outjson, args):
    logging.info("Analyzing results")
    findings = []
    f = open(outjson, 'r')
    fdict = json.loads(f.read())
    issues = fdict['issues']
    for i in issues:
        idict = {}
        idict['asset_id'] = args.assetid
        idict['twc_id'] = 'ARACHNI-' + hashlib.md5(str(i['name']).encode('utf-8')).hexdigest()
        idict['twc_title'] = i['name']
        idict['rating'] = get_rating(i['severity'])
        idict['object_id'] = i['page']['dom']['url'] 
        idict['type'] = 'DAST'
        idict['object_meta'] = ''
        idict['details'] = i['description'] 
        findings.append(idict)
    return findings

def run(args):
    arachni_base = args.pluginpath
    arachni = arachni_base + '/arachni'
    arachni_reporter = arachni_base + '/arachni_reporter'

    if not os.path.isfile(arachni) or not os.access(arachni, os.X_OK):
        logging.error('arachni CLI not found')
        sys.exit(1) 
    if not os.path.isfile(arachni_reporter) or not os.access(arachni_reporter, os.X_OK):
        logging.error('arachni CLI not found')
        sys.exit(1) 

    logging.info("Running arachni plugin. This could take a while")
    path = tempfile.mkdtemp()
    outafr = path+"/"+args.assetid+".afr"
    outjson = path+"/"+args.assetid+".json"
    rparams = " --report-save-path "+outafr
    logging.info("Using reporting options: "+rparams)
    logging.info("Please do not override the reporting options")

    params = args.args
    if params != None:
        params = " " + params + " " + rparams + " " + args.url
    else:
        params = " " + rparams + " " + args.url
    #logging.info("arachni command line: "+arachni+ " " + params)
    cmdarr = [arachni + " " + params]
    try:
        out = subprocess.check_output(cmdarr, shell=True)
    except subprocess.CalledProcessError:
        logging.error("Error running arachni CLI")
        shutil.rmtree(path, onerror = on_rm_error)
        return None 
    logging.info("arachni run completed")

    reporter_cmd = arachni_reporter + " " +outafr+" "+" --report=json:outfile="+outjson
    #logging.info("arachni_reporter command line: "+reporter_cmd)
    cmdarr = [reporter_cmd]
    try:
        out = subprocess.check_output(cmdarr, shell=True)
    except subprocess.CalledProcessError:
        logging.error("Error running arachni_reporter CLI")
        shutil.rmtree(path, onerror = on_rm_error)
        return None 
    logging.info("arachni_reporter run completed")

    findings = parse_arachni(outjson, args)
    shutil.rmtree(path, onerror = on_rm_error)

    return findings



import sys
import os
import logging
import csv
from .azure_cis_tool import azure_cis as azure_cis_tool

def get_issues_from_csv_file(config_issues_csv, assetid):
    findings = []
    with open(config_issues_csv, "r") as csv_file:
        csv_reader = csv.DictReader(csv_file)
        for row in csv_reader:
            if 'Failed' not in row['RESULT']:
                continue
            issue = { }
            issue['twc_id'] = 'cis-azure-bench-check-'+row['CHECK'].split()[0]
            issue['asset_id'] = assetid
            issue['twc_title'] = row['CHECK']
            issue['details'] = row['DETAILS']
            issue['type'] = 'Azure CIS'
            issue['rating'] = '4'
            issue['object_id'] = row['SUBSCRIPTION']
            issue['object_meta'] = ''
            findings.append(issue)
    os.remove(config_issues_csv)
    return findings

def run_azure_cis_bench(args):
    if args.assetid.strip() == "":
        logging.error("[assetid] cannot be empty")
        sys.exit(1)
    config_issues_csv = azure_cis_tool.run_tests()
    if config_issues_csv is None:
        sys.exit(1)
    asset = { }
    asset['id'] = args.assetid
    asset['name'] = args.assetname if args.assetname and args.assetname.strip() != "" else asset['id']
    asset['type'] = 'Azure'
    asset['owner'] = args.handle
    asset['products'] = []
    asset['tags'] = ['Azure', 'CIS']
    asset['config_issues'] = get_issues_from_csv_file(config_issues_csv, asset['id'])
    args.no_scan = True
    return asset

def get_inventory(args):
    asset = run_azure_cis_bench(args)
    return [ asset ]



#!/usr/bin/env python
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.

import sys
import os
import logging
import argparse
import time
import json
import traceback
import pkg_resources
import pkgutil
import importlib
import getpass
from os.path import expanduser
import warnings
with warnings.catch_warnings():
   warnings.simplefilter("ignore", category=Warning)
   from cryptography.hazmat.backends import default_backend
   from cryptography.hazmat.primitives import hashes
   from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
   from cryptography.fernet import Fernet

try:
    from . import utils
    from . import aws
    from . import linux
    from . import repo
    from . import docker
    from . import kubernetes
    from . import azure
    from . import acr
    from . import ecr
    from . import gcp
    from . import gcr
    from . import servicenow
    from . import sbom
    from . import inv_file
    from . import fingerprint
    from . import dast
    from . import docker_cis
    from . import aws_cis
    from . import azure_cis
    from . import azure_functions
    from . import gcp_cis
    from . import k8s_cis
    from . import gcloud_functions
    from . import vmware 
    from . import policy as policy_lib
    from .__init__ import __version__
except (ImportError,ValueError):
    from twigs import aws
    from twigs import linux
    from twigs import repo
    from twigs import docker
    from twigs import kubernetes
    from twigs import azure
    from twigs import acr
    from twigs import ecr
    from twigs import gcp
    from twigs import gcr
    from twigs import servicenow
    from twigs import sbom
    from twigs import inv_file
    from twigs import fingerprint
    from twigs import dast
    from twigs import docker_cis
    from twigs import aws_cis
    from twigs import azure_cis
    from twigs import azure_functions
    from twigs import gcp_cis
    from twigs import k8s_cis
    from twigs import gcloud_functions
    from twigs import utils
    from twigs import policy as policy_lib
    from twigs.__init__ import __version__

def export_assets_to_file(assets, json_file):
    logging.info("Exporting assets to JSON file [%s]", json_file)
    with open(json_file, "w") as fd:
        json.dump(assets, fd, indent=2, sort_keys=True)
    logging.info("Successfully exported assets to JSON file!")

def push_asset_to_TW(asset, args):
    asset_url = "https://" + args.instance + "/api/v2/assets/"
    auth_data = "?handle=" + args.handle + "&token=" + args.token + "&format=json"
    if args.email_report:
        auth_data = auth_data + "&esr=true" # email secrets report (esr)
    asset_id = asset['id']

    resp = utils.requests_get(asset_url + asset_id + "/" + auth_data)
    if resp is None:
        logging.info("Unable to check if asset exists...skipping asset [%s]", asset_id)
        return None, False
    if resp.status_code != 200:
        logging.info("Creating new asset [%s]", asset_id)
        # Asset does not exist so create one with POST
        resp = utils.requests_post(asset_url + auth_data, json=asset)
        if resp is not None and resp.status_code == 200:
            logging.info("Successfully created new asset [%s]", asset_id)
            logging.info("Response content: %s", resp.content.decode(args.encoding))
            return asset_id, True
        else:
            logging.error("Failed to create new asset [%s]", asset_id)
            if resp is not None:
                logging.error("Response details: %s", resp.content.decode(args.encoding))
            return None, False
    else:
        logging.info("Updating asset [%s]", asset_id)
        # asset exists so update it with PUT
        resp = utils.requests_put(asset_url + asset_id + "/" + auth_data, json=asset)
        if resp is not None and resp.status_code == 200:
            logging.info("Successfully updated asset [%s]", asset_id)
            logging.debug("Response content: %s", resp.content.decode(args.encoding))
            resp_json = resp.json()
            if 'No product updates' in resp_json['status']:
                return asset_id, False
            return asset_id, True
        else:
            logging.error("Failed to update existing asset [%s]", asset_id)
            if resp is not None:
                logging.error("Response details: %s", resp.content.decode(args.encoding))
            return None, False

def push_assets_to_TW(assets, args):
    asset_id_list = []
    scan_asset_id_list = []
    for asset in assets:
        asset_id, scan = push_asset_to_TW(asset, args)
        if asset_id is not None:
            asset_id_list.append(asset_id)
        if scan:
            scan_asset_id_list.append(asset_id)
    return asset_id_list, scan_asset_id_list

def run_scan(asset_id_list, pj_json, args):
    if args.no_scan is not True:
        if len(asset_id_list) == 0:
            logging.debug("No asset scan required")
            return 
        run_va_scan = True
        run_lic_scan = True
        if pj_json is not None:
            for policy in pj_json['policy_json']:
                if policy['type'] == 'vulnerability':
                    logging.info("Impact assessment performed as part of policy evaluation")
                    run_va_scan = False # VA scan already done, so don't do it again
                elif policy['type'] == 'license':
                    logging.info("License compliance performed as part of policy evaluation")
                    run_lic_scan = False # License scan already done, so don't do it again

        scan_api_url = "https://" + args.instance + "/api/v1/scans/?handle=" + args.handle + "&token=" + args.token + "&format=json"
        if run_va_scan:
            # Start VA
            scan_payload = { }
            scan_payload['scan_type'] = 'full' 
            scan_payload['assets'] = asset_id_list
            # if args.purge_assets:
            #    scan_payload['mode'] = 'email-purge'
            if args.email_report:
                scan_payload['mode'] = 'email'
            resp = utils.requests_post(scan_api_url, json=scan_payload)
            if resp is not None and resp.status_code == 200:
                logging.info("Started impact refresh")
            else:
                logging.error("Failed to start impact refresh")
                if resp is not None:
                    logging.error("Response details: %s", resp.content.decode(args.encoding))
        if run_lic_scan and (args.mode == "repo" or args.mode == "file_repo" or args.mode == "sbom"):
            # Start license compliance assessment
            scan_payload = { }
            scan_payload['assets'] = asset_id_list
            scan_payload['license_scan'] = True
            # if args.purge_assets:
            #    scan_payload['mode'] = 'email-purge'
            if args.email_report:
                scan_payload['mode'] = 'email'
            resp = utils.requests_post(scan_api_url, json=scan_payload)
            if resp is not None and resp.status_code == 200:
                logging.info("Started license compliance assessment")
            else:
                logging.error("Failed to start license compliance assessment")
                if resp is not None:
                    logging.error("Response details: %s", resp.content.decode(args.encoding))

def remove_standard_tags(assets):
    for asset in assets:
        existing_tags = asset['tags']
        if existing_tags is not None:
            new_tags = []
            for et in existing_tags:
                if et.split(':')[0] in utils.SYSTEM_TAGS:
                    new_tags.append(et)
            asset['tags'] = new_tags

def add_asset_tags(assets, tags):
    for asset in assets:
        existing_tags = asset.get('tags')
        if existing_tags is None:
            asset['tags'] = tags
        else:
            existing_tags.extend(tags)

def add_asset_owners(assets, additional_owners):
    for asset in assets:
        asset['notify'] = additional_owners

def add_asset_location(assets, location):
    for asset in assets:
        asset['location'] = location

def add_asset_criticality_tag(assets, asset_criticality):
    asset_criticality_tag = 'CRITICALITY:'+str(asset_criticality)
    add_asset_tags(assets, [asset_criticality_tag])

def sub_pkg_get_inventory(args):
    dist = "twigs_" + args.mode
    try:
        pkg_resources.get_distribution(dist)
    except pkg_resources.DistributionNotFound:
        logging.error("Error required package [%s] is not installed", dist)
        logging.error("Please install using command [sudo pip install %s]", dist)
        sys.exit(1)
    module = importlib.import_module('%s.%s' % (dist,dist))
    get_inventory_func = getattr(module, "get_inventory")
    assets = get_inventory_func(args)
    return assets

def authenticate_user(tw_user, tw_pwd, tw_instance):
    payload = { }
    payload["handle"] = tw_user
    payload["password"] = tw_pwd
    twigs_auth_url = "https://" + tw_instance + "/api/v1/twigsauth/"
    resp = utils.requests_post(twigs_auth_url, json=payload)
    if resp is not None and resp.status_code == 200:
        logging.info("User logged in successfully")
        return resp.json()["token"]
    else:
        logging.error("User authentication failed")
        if resp is not None:
            logging.debug("Response: %s", resp.content)
        sys.exit(1)

def login_user(args):
    try:
        if sys.version_info.major < 3:
            tw_user = raw_input("Enter email of ThreatWorx user: ")
        else:
            tw_user = input("Enter email of ThreatWorx user: ")
        temp_pwd = getpass.getpass(prompt='Enter password: ')
        if sys.version_info.major < 3:
            tw_instance = raw_input("Enter ThreatWorx instance [threatworx.io]: ")
        else:
            tw_instance = input("Enter ThreatWorx instance [threatworx.io]: ")
    except KeyboardInterrupt:
        print("")
        sys.exit(1)
    if tw_instance == "":
        tw_instance = "threatworx.io"
    tw_token = authenticate_user(tw_user, temp_pwd, tw_instance)
    auth_dict = {}
    auth_dict['handle'] = tw_user
    auth_dict['token'] = tw_token
    auth_dict['instance'] = tw_instance
    user_home_dir = expanduser("~")
    tw_dir = user_home_dir + os.path.sep + '.tw'
    if os.path.isdir(tw_dir) == False:
        os.mkdir(tw_dir, 0o700)
    auth_file = tw_dir + os.path.sep + 'auth.json'
    if os.path.exists(auth_file):
        os.remove(auth_file)
    with open(auth_file, "w") as fd:
        json.dump(auth_dict, fd)
    os.chmod(auth_file, 0o600)
    sys.exit(0)

def logout_user(args):
    user_home_dir = expanduser("~")
    tw_dir = user_home_dir + os.path.sep + '.tw'
    if os.path.isdir(tw_dir):
        auth_file = tw_dir + os.path.sep + 'auth.json'
        if os.path.exists(auth_file):
            os.remove(auth_file)
    sys.exit(0)

def get_logged_in_user_details():
    ret_dict = { }
    user_home_dir = expanduser("~")
    tw_dir = user_home_dir + os.path.sep + '.tw'
    if os.path.isdir(tw_dir):
        auth_file = tw_dir + os.path.sep + 'auth.json'
        if os.path.exists(auth_file):
            with open(auth_file, "r") as fd:
                ret_dict = json.load(fd)
    return ret_dict

def main(args=None):

    try:
    
        if args is None:
            args = sys.argv[1:]

        if sys.platform != 'win32':
            utils.set_requests_verify(os.path.dirname(os.path.realpath(__file__)) + os.sep + 'gd-ca-bundle.crt')
        logfilename = "twigs.log"
        logging_level = logging.WARN

        parser = argparse.ArgumentParser(description='ThreatWorx Information Gathering Script (twigs) to discover assets like hosts, cloud instances, containers and source code repositories')
        subparsers = parser.add_subparsers(title="modes", description="Discovery modes and commands supported", dest="mode")
        # Required arguments
        parser.add_argument('--version', action='version', version='%(prog)s ' + __version__)
        parser.add_argument('--handle', help='The ThreatWorx registered email of the user. Note this can set as "TW_HANDLE" environment variable', required=False)
        parser.add_argument('--token', help='The ThreatWorx API token of the user. Note this can be set as "TW_TOKEN" environment variable', required=False)
        parser.add_argument('--instance', help='The ThreatWorx instance. Note this can be set as "TW_INSTANCE" environment variable')
        parser.add_argument('--location', help='Specify location information for discovered asset(s).')
        parser.add_argument('--create_empty_asset', action='store_true', help='Create empty asset even if nothing is discovered. Applicable to source code (repo) assets.')
        parser.add_argument('--tag_critical', action='store_true', help='Tag the discovered asset(s) as critical')
        parser.add_argument('--tag', action='append', help='Add specified tag to discovered asset(s). You can specify this option multiple times to add multiple tags')
        parser.add_argument('--owner', action='append', help='Add additional owner(s) to discovered asset(s). You can specify this option multiple times to add multiple owners. Note user discovering the asset is added as owner by default')
        parser.add_argument('--no_auto_tags', action='store_true', help='Disable auto tagging of assets with standard classification tags. Only user specified tags will be applied')
        #parser.add_argument('--asset_criticality', choices=['1', '2', '3','4', '5'], help='Business criticality of the discovered assets on a scale of 1 (low) to 5 (high).', required=False)
        parser.add_argument('--apply_policy', help='One or more policy names as a comma-separated list', required=False)
        parser.add_argument('--out', help='Specify name of the JSON file to hold the exported asset information.')
        parser.add_argument('--no_scan', action='store_true', help='Do not initiate a baseline assessment')
        parser.add_argument('--email_report', action='store_true', help='After impact refresh is complete email scan report to self')
        group = parser.add_mutually_exclusive_group()
        group.add_argument('-q','--quiet', action='store_true', help='Disable verbose logging')
        group.add_argument('-v','--verbosity', action='count', default=0, help='Specify the verbosity level. Use multiple times to increase verbosity level')
        if sys.platform != 'win32':
            parser.add_argument('--schedule', help='Run this twigs command at specified schedule (crontab format)')
        parser.add_argument('--encoding', help='Specify the encoding. Default is "latin-1"', default='latin-1')
        parser.add_argument('--insecure', action='store_true', help=argparse.SUPPRESS)
        # parser.add_argument('--purge_assets', action='store_true', help='Purge the asset(s) after impact refresh is complete and scan report is emailed to self')


        # Arguments required for Login
        parser_login = subparsers.add_parser("login", help = "Login to twigs")

        # Arguments required for Logout
        parser_logout = subparsers.add_parser("logout", help = "Logout from twigs")

        # Arguments required for AWS discovery
        parser_aws = subparsers.add_parser ("aws", help = "Discover AWS instances")
        parser_aws.add_argument('--aws_account', help='AWS account ID', required=True)
        parser_aws.add_argument('--aws_access_key', help='AWS access key', required=True)
        parser_aws.add_argument('--aws_secret_key', help='AWS secret key', required=True)
        parser_aws.add_argument('--aws_region', help='AWS region', required=True)
        parser_aws.add_argument('--aws_s3_bucket', help='AWS S3 inventory bucket', required=True)
        parser_aws.add_argument('--enable_tracking_tags', action='store_true', help='Enable recording AWS specific information (like AWS Account ID, etc.) as asset tags', required=False)
        
        # Arguments required for Azure discovery
        parser_azure = subparsers.add_parser ("azure", help = "Discover Azure instances")
        parser_azure.add_argument('--azure_tenant_id', help='Azure Tenant ID', required=True)
        parser_azure.add_argument('--azure_application_id', help='Azure Application ID', required=True)
        parser_azure.add_argument('--azure_application_key', help='Azure Application Key', required=True)
        parser_azure.add_argument('--azure_subscription', help='Azure Subscription. If not specified, then available values will be displayed', required=False)
        parser_azure.add_argument('--azure_resource_group', help='Azure Resource Group. If not specified, then available values will be displayed', required=False)
        parser_azure.add_argument('--azure_workspace', help='Azure Workspace ID. If not specified, then available values will be displayed', required=False)
        parser_azure.add_argument('--enable_tracking_tags', action='store_true', help='Enable recording Azure specific information (like Azure Tenant ID, etc.) as asset tags', required=False)

        # Arguments required for Google Cloud Platform discovery
        parser_gcp = subparsers.add_parser ("gcp", help = "Discover Google Cloud Platform instances")
        parser_gcp.add_argument('--enable_tracking_tags', action='store_true', help='Enable recording GCP specific information (like Project ID, etc.) as asset tags', required=False)

        # Arguments required for AWS Container Registry discovery 
        parser_ecr = subparsers.add_parser ("ecr", help = "Discover AWS Container Registry images")
        parser_ecr.add_argument('--registry', help='The AWS Container Registry (AWS account ID) which needs to be inspected for all repositories.')
        parser_ecr.add_argument('--image', help='The fully qualified image name (repositoryUri with optional tag) to be inspected. If tag is not given, latest will be determined for all images under this repository')
        parser_ecr.add_argument ("--repository_type", help = "Specify repository type (public/private). Defaults to private repositories if not specified", choices=['public','private'], default='private')
        parser_ecr.add_argument('--tmp_dir', help='Temporary directory. Defaults to /tmp', required=False)

        parser_ecr.add_argument('--containerid', help=argparse.SUPPRESS, required=False)
        parser_ecr.add_argument('--assetid', help=argparse.SUPPRESS, required=False)
        parser_ecr.add_argument('--assetname', help=argparse.SUPPRESS, required=False)
        parser_ecr.add_argument('--start_instance', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--repo', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--type', choices=repo.SUPPORTED_TYPES, help=argparse.SUPPRESS)
        parser_ecr.add_argument('--level', help=argparse.SUPPRESS, choices=['shallow','deep'], default='shallow')
        parser_ecr.add_argument('--secrets_scan', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--enable_entropy', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--regex_rules_file', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--check_common_passwords', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--common_passwords_file', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--include_patterns', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--include_patterns_file', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--exclude_patterns', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--exclude_patterns_file', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--mask_secret', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--no_code', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--sast', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--iac_checks', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--check_vuln', action='append', help='Run plugin to detect impact of specified vulnerabilities. You can use this option multiple times to specify multiple vulnerabilities')
        parser_ecr.add_argument('--check_all_vulns', action='store_true', help='Run plugins to detect impact of all vulnerabilities')

        # Arguments required for Azure Container Registry container discovery 
        parser_acr = subparsers.add_parser ("acr", help = "Discover Azure Container Registry images")
        parser_acr.add_argument('--registry', help='The Azure Container Registry which needs to be inspected.')
        parser_acr.add_argument('--image', help='The fully qualified image name (with tag) which needs to be inspected. If tag is not given, latest will be determined and used.')
        parser_acr.add_argument('--tmp_dir', help='Temporary directory. Defaults to /tmp', required=False)
        parser_acr.add_argument('--containerid', help=argparse.SUPPRESS, required=False)
        parser_acr.add_argument('--assetid', help=argparse.SUPPRESS, required=False)
        parser_acr.add_argument('--assetname', help=argparse.SUPPRESS, required=False)
        parser_acr.add_argument('--start_instance', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--repo', help=argparse.SUPPRESS)
        parser_acr.add_argument('--type', choices=repo.SUPPORTED_TYPES, help=argparse.SUPPRESS)
        parser_acr.add_argument('--level', help=argparse.SUPPRESS, choices=['shallow','deep'], default='shallow')
        parser_acr.add_argument('--secrets_scan', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--enable_entropy', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--regex_rules_file', help=argparse.SUPPRESS)
        parser_acr.add_argument('--check_common_passwords', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--common_passwords_file', help=argparse.SUPPRESS)
        parser_acr.add_argument('--include_patterns', help=argparse.SUPPRESS)
        parser_acr.add_argument('--include_patterns_file', help=argparse.SUPPRESS)
        parser_acr.add_argument('--exclude_patterns', help=argparse.SUPPRESS)
        parser_acr.add_argument('--exclude_patterns_file', help=argparse.SUPPRESS)
        parser_acr.add_argument('--mask_secret', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--no_code', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--sast', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--iac_checks', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--check_vuln', action='append', help='Run plugin to detect impact of specified vulnerabilities. You can use this option multiple times to specify multiple vulnerabilities')
        parser_acr.add_argument('--check_all_vulns', action='store_true', help='Run plugins to detect impact of all vulnerabilities')

        # Arguments required for Google Cloud Registry container discovery 
        parser_gcr = subparsers.add_parser ("gcr", help = "Discover Google Cloud Registry images")
        parser_gcr.add_argument('--repository', help='The GCR image respository url which needs to be inspected.')
        parser_gcr.add_argument('--image', help='The fully qualified image name (with tag / digest) which needs to be inspected. If tag / digest is not given, latest will be determined and used.')
        parser_gcr.add_argument('--tmp_dir', help='Temporary directory. Defaults to /tmp', required=False)
        parser_gcr.add_argument('--containerid', help=argparse.SUPPRESS, required=False)
        parser_gcr.add_argument('--assetid', help=argparse.SUPPRESS, required=False)
        parser_gcr.add_argument('--assetname', help=argparse.SUPPRESS, required=False)
        parser_gcr.add_argument('--start_instance', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--repo', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--type', choices=repo.SUPPORTED_TYPES, help=argparse.SUPPRESS)
        parser_gcr.add_argument('--level', help=argparse.SUPPRESS, choices=['shallow','deep'], default='shallow')
        parser_gcr.add_argument('--secrets_scan', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--enable_entropy', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--regex_rules_file', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--check_common_passwords', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--common_passwords_file', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--include_patterns', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--include_patterns_file', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--exclude_patterns', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--exclude_patterns_file', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--mask_secret', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--no_code', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--sast', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--iac_checks', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--check_vuln', action='append', help='Run plugin to detect impact of specified vulnerabilities. You can use this option multiple times to specify multiple vulnerabilities')
        parser_gcr.add_argument('--check_all_vulns', action='store_true', help='Run plugins to detect impact of all vulnerabilities')

        # Arguments required for docker discovery 
        parser_docker = subparsers.add_parser ("docker", help = "Discover docker instances")
        parser_docker.add_argument('--image', help='The docker image (repo:tag) which needs to be inspected. If tag is not given, "latest" will be assumed.')
        parser_docker.add_argument('--containerid', help='The container ID of a running docker container which needs to be inspected.')
        parser_docker.add_argument('--assetid', help=argparse.SUPPRESS)
        parser_docker.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_docker.add_argument('--tmp_dir', help='Temporary directory. Defaults to /tmp', default='/tmp')
        parser_docker.add_argument('--start_instance', action='store_true', help='If image inventory fails, try starting a container instance to inventory contents. Use with caution', required=False)
        parser_docker.add_argument('--repo', help=argparse.SUPPRESS)
        parser_docker.add_argument('--type', choices=repo.SUPPORTED_TYPES, help=argparse.SUPPRESS)
        parser_docker.add_argument('--level', help=argparse.SUPPRESS, choices=['shallow','deep'], default='shallow')
        parser_docker.add_argument('--secrets_scan', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--enable_entropy', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--regex_rules_file', help=argparse.SUPPRESS)
        parser_docker.add_argument('--check_common_passwords', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--common_passwords_file', help=argparse.SUPPRESS)
        parser_docker.add_argument('--include_patterns', help=argparse.SUPPRESS)
        parser_docker.add_argument('--include_patterns_file', help=argparse.SUPPRESS)
        parser_docker.add_argument('--exclude_patterns', help=argparse.SUPPRESS)
        parser_docker.add_argument('--exclude_patterns_file', help=argparse.SUPPRESS)
        parser_docker.add_argument('--mask_secret', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--no_code', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--sast', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--iac_checks', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--check_vuln', action='append', help='Run plugin to detect impact of specified vulnerabilities. You can use this option multiple times to specify multiple vulnerabilities')
        parser_docker.add_argument('--check_all_vulns', action='store_true', help='Run plugins to detect impact of all vulnerabilities')

        # Arguments required for Kubernetes discovery
        parser_k8s = subparsers.add_parser ("k8s", help = "Discover Kubernetes environment")
        group = parser_k8s.add_mutually_exclusive_group(required=True)
        group.add_argument('--deployment_yaml', help='Path to Kubernetes deployment manifest definition YAML file.')
        group.add_argument('--helm_chart', help='Specify the helm chart (folder, repo/chartname).')
        parser_k8s.add_argument('--tmp_dir', help='Temporary directory. Defaults to /tmp', required=False)
        parser_k8s.add_argument('--containerid', help=argparse.SUPPRESS, required=False)
        parser_k8s.add_argument('--assetid', help=argparse.SUPPRESS, required=False)
        parser_k8s.add_argument('--assetname', help=argparse.SUPPRESS, required=False)
        parser_k8s.add_argument('--start_instance', action='store_true', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--repo', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--type', choices=repo.SUPPORTED_TYPES, help=argparse.SUPPRESS)
        parser_k8s.add_argument('--level', help=argparse.SUPPRESS, choices=['shallow','deep'], default='shallow')
        parser_k8s.add_argument('--secrets_scan', action='store_true', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--enable_entropy', action='store_true', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--regex_rules_file', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--check_common_passwords', action='store_true', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--common_passwords_file', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--include_patterns', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--include_patterns_file', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--exclude_patterns', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--exclude_patterns_file', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--mask_secret', action='store_true', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--no_code', action='store_true', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--sast', action='store_true', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--iac_checks', action='store_true', help=argparse.SUPPRESS)
        parser_k8s.add_argument('--check_vuln', action='append', help='Run plugin to detect impact of specified vulnerabilities. You can use this option multiple times to specify multiple vulnerabilities')
        parser_k8s.add_argument('--check_all_vulns', action='store_true', help='Run plugins to detect impact of all vulnerabilities')

        # Arguments required for Repo discovery
        parser_repo = subparsers.add_parser ("repo", help = "Discover source code repository as asset")
        parser_repo.add_argument('--repo', help='Local path or git repo url for project', required=True)
        parser_repo.add_argument('--branch', help='Optional branch of remote git repo')
        parser_repo.add_argument('--type', choices=repo.SUPPORTED_TYPES, help='Type of open source component to scan for. Defaults to all supported types if not specified', required=False)
        parser_repo.add_argument('--level', help='Possible values {shallow, deep}. Shallow restricts discovery to 1st level dependencies only. Deep discovers dependencies at all levels. Defaults to shallow discovery if not specified', choices=['shallow','deep'], required=False, default='shallow')
        parser_repo.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset')
        parser_repo.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        # Switches related to secrets scan for repo
        parser_repo.add_argument('--secrets_scan', action='store_true', help='Perform a scan to look for secrets in the code')
        parser_repo.add_argument('--enable_entropy', action='store_true', help='Identify entropy based secrets')
        parser_repo.add_argument('--regex_rules_file', help='Path to JSON file specifying regex rules')
        parser_repo.add_argument('--check_common_passwords', action='store_true', help='Look for top common passwords.')
        parser_repo.add_argument('--common_passwords_file', help='Specify your own common passwords file. One password per line in file')
        parser_repo.add_argument('--include_patterns', help='Specify patterns which indicate files to be included in the secrets scan. Separate multiple patterns with comma.')
        parser_repo.add_argument('--include_patterns_file', help='Specify file containing include patterns which indicate files to be included in the secrets scan. One pattern per line in file.')
        parser_repo.add_argument('--exclude_patterns', help='Specify patterns which indicate files to be excluded in the secrets scan. Separate multiple patterns with comma.')
        parser_repo.add_argument('--exclude_patterns_file', help='Specify file containing exclude patterns which indicate files to be excluded in the secrets scan. One pattern per line in file.')
        parser_repo.add_argument('--mask_secret', action='store_true', help='Mask identified secret before storing for reference in ThreatWorx.')
        parser_repo.add_argument('--no_code', action='store_true', help='Disable storing code for reference in ThreatWorx.')
        parser_repo.add_argument('--sast', action='store_true', help='Perform static code analysis on your source code')
        parser_repo.add_argument('--iac_checks', action='store_true', help='Perform security checks on IaC templates')


        # Arguments required for Azure Functions 
        parser_az_functions = subparsers.add_parser("azure_functions", help = "Discover and scan Azure Functions soure code")
        parser_az_functions.add_argument('--repo', help=argparse.SUPPRESS)
        parser_az_functions.add_argument('--type', help=argparse.SUPPRESS)
        parser_az_functions.add_argument('--level', help=argparse.SUPPRESS, default='shallow')
        parser_az_functions.add_argument('--assetid', help=argparse.SUPPRESS)
        parser_az_functions.add_argument('--assetname', help=argparse.SUPPRESS)
        # Switches related to secrets scan for repo
        parser_az_functions.add_argument('--secrets_scan', action='store_true', help='Perform a scan to look for secrets in the code')
        parser_az_functions.add_argument('--enable_entropy', action='store_true', help='Identify entropy based secrets')
        parser_az_functions.add_argument('--regex_rules_file', help='Path to JSON file specifying regex rules')
        parser_az_functions.add_argument('--check_common_passwords', action='store_true', help='Look for top common passwords.')
        parser_az_functions.add_argument('--common_passwords_file', help='Specify your own common passwords file. One password per line in file')
        parser_az_functions.add_argument('--include_patterns', help='Specify patterns which indicate files to be included in the secrets scan. Separate multiple patterns with comma.')
        parser_az_functions.add_argument('--include_patterns_file', help='Specify file containing include patterns which indicate files to be included in the secrets scan. One pattern per line in file.')
        parser_az_functions.add_argument('--exclude_patterns', help='Specify patterns which indicate files to be excluded in the secrets scan. Separate multiple patterns with comma.')
        parser_az_functions.add_argument('--exclude_patterns_file', help='Specify file containing exclude patterns which indicate files to be excluded in the secrets scan. One pattern per line in file.')
        parser_az_functions.add_argument('--mask_secret', action='store_true', help='Mask identified secret before storing for reference in ThreatWorx.')
        parser_az_functions.add_argument('--no_code', action='store_true', help='Disable storing code for reference in ThreatWorx.')
        parser_az_functions.add_argument('--sast', action='store_true', help='Perform static code analysis on your source code')
        parser_az_functions.add_argument('--iac_checks', action='store_true', help='Perform security checks on IaC templates')

        # Arguments required for Google Cloud Functions 
        parser_gcloud_functions = subparsers.add_parser("gcloud_functions", help = "Discover and scan Google Cloud Functions soure code")
        parser_gcloud_functions.add_argument('--projects', help='A comma separated list of GCP project IDs', required=True)
        parser_gcloud_functions.add_argument('--repo', help=argparse.SUPPRESS)
        parser_gcloud_functions.add_argument('--type', help=argparse.SUPPRESS)
        parser_gcloud_functions.add_argument('--level', help=argparse.SUPPRESS, default='shallow')
        parser_gcloud_functions.add_argument('--assetid', help=argparse.SUPPRESS)
        parser_gcloud_functions.add_argument('--assetname', help=argparse.SUPPRESS)
        parser_gcloud_functions.add_argument('--secrets_scan', action='store_true', help='Perform a scan to look for secrets in the code')
        parser_gcloud_functions.add_argument('--enable_entropy', action='store_true', help='Identify entropy based secrets')
        parser_gcloud_functions.add_argument('--regex_rules_file', help='Path to JSON file specifying regex rules')
        parser_gcloud_functions.add_argument('--check_common_passwords', action='store_true', help='Look for top common passwords.')
        parser_gcloud_functions.add_argument('--common_passwords_file', help='Specify your own common passwords file. One password per line in file')
        parser_gcloud_functions.add_argument('--include_patterns', help='Specify patterns which indicate files to be included in the secrets scan. Separate multiple patterns with comma.')
        parser_gcloud_functions.add_argument('--include_patterns_file', help='Specify file containing include patterns which indicate files to be included in the secrets scan. One pattern per line in file.')
        parser_gcloud_functions.add_argument('--exclude_patterns', help='Specify patterns which indicate files to be excluded in the secrets scan. Separate multiple patterns with comma.')
        parser_gcloud_functions.add_argument('--exclude_patterns_file', help='Specify file containing exclude patterns which indicate files to be excluded in the secrets scan. One pattern per line in file.')
        parser_gcloud_functions.add_argument('--mask_secret', action='store_true', help='Mask identified secret before storing for reference in ThreatWorx.')
        parser_gcloud_functions.add_argument('--no_code', action='store_true', help='Disable storing code for reference in ThreatWorx.')
        parser_gcloud_functions.add_argument('--sast', action='store_true', help='Perform static code analysis on your source code')
        parser_gcloud_functions.add_argument('--iac_checks', action='store_true', help='Perform security checks on IaC templates')

        # Arguments required for Host discovery on Linux
        parser_linux = subparsers.add_parser ("host", help = "Discover linux host assets")
        parser_linux.add_argument('--remote_hosts_csv', help='CSV file containing details of remote hosts. CSV file column header [1st row] should be: hostname,userlogin,userpwd,privatekey,assetid,assetname. Note "hostname" column can contain hostname, IP address, CIDR range.')
        parser_linux.add_argument('--host_list', help='Same as the option: remote_hosts_csv. A file (currently in CSV format) containing details of remote hosts. CSV file column header [1st row] should be: hostname,userlogin,userpwd,privatekey,assetid,assetname. Note "hostname" column can contain hostname, IP address, CIDR range.')
        parser_linux.add_argument('--secure', action='store_true', help='Use this option to encrypt clear text passwords in the host list file')
        parser_linux.add_argument('--password', help='A password used to encrypt / decrypt login information from the host list file')
        parser_linux.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset')
        parser_linux.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_linux.add_argument('--no_ssh_audit', action='store_true', help='Skip ssh audit')
        parser_linux.add_argument('--no_host_benchmark', action='store_true', help='Skip host benchmark audit')
        parser_linux.add_argument('--check_vuln', action='append', help='Run plugin to detect impact of specified vulnerabilities. You can use this option multiple times to specify multiple vulnerabilities')
        parser_linux.add_argument('--check_all_vulns', action='store_true', help='Run plugins to detect impact of all vulnerabilities')

        # Arguments required for vmware discovery
        parser_vmware = subparsers.add_parser ("vmware", help = "Discover VMware vCenter/ESX assets")
        parser_vmware.add_argument('--host', help='A vCenter host name or IP', required=True)
        parser_vmware.add_argument('--user', help='A vCenter user name', required=True)
        parser_vmware.add_argument('--password', help='Password for the vCenter user. Note this can be set as "VCENTER_PASSWD" environment variable')

        # Arguments required for nmap discovery
        parser_nmap = subparsers.add_parser ("nmap", help = "Discover assets using nmap")
        parser_nmap.add_argument('--hosts', help='A hostname, IP address or CIDR range', required=True)
        parser_nmap.add_argument('--no_ssh_audit', action='store_true', help='Skip ssh audit')

        # Arguments required for SBOM-based discovery
        parser_sbom = subparsers.add_parser("sbom", help = "Ingest asset inventory from SBOM (Software Bill Of Materials)")
        parser_sbom.add_argument('--input', help='Absolute path to SBOM artifact', required=True)
        parser_sbom.add_argument('--standard', choices=sbom.SUPPORTED_SBOM_STANDARDS, help='Specifies SBOM standard.', required=True)
        all_formats = set()
        for std in sbom.SUPPORTED_SBOM_FORMATS_FOR_STANDARD:
            for f in sbom.SUPPORTED_SBOM_FORMATS_FOR_STANDARD[std]:
                all_formats.add(f)
        all_formats = list(all_formats)
        parser_sbom.add_argument('--format', choices=all_formats, help='Specifies format of SBOM artifact.', required=True)
        parser_sbom.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset', required=False)
        parser_sbom.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')

        # Arguments required for File-based discovery
        parser_file = subparsers.add_parser ("file", help = "Ingest asset inventory from file")
        parser_file.add_argument('--input', help='Absolute path to single input inventory file or a directory containing JSON or CSV files. Supported file formats are: CSV, JSON & PDF', required=True)
        parser_file.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset. Defaults to input filename if not specified. Applies only for PDF files.')
        parser_file.add_argument('--assetname', help='A name/label to be assigned to the discovered asset. Defaults to assetid is not specified. Applies only for PDF files.')
        parser_file.add_argument('--type', choices=['repo'], help='Type of asset. Defaults to repo if not specified. Applies only for PDF files.', required=False, default='repo')

        # Arguments required for ServiceNow discovery
        parser_snow = subparsers.add_parser ("servicenow", help = "Ingest inventory from ServiceNow CMDB")
        parser_snow.add_argument('--snow_user', help='User name of ServiceNow account', required=True)
        parser_snow.add_argument('--snow_user_pwd', help='User password of ServiceNow account', required=True)
        parser_snow.add_argument('--snow_instance', help='ServiceNow Instance name', required=True)
        parser_snow.add_argument('--enable_tracking_tags', action='store_true', help='Enable recording ServiceNow specific information (like ServiceNow instance name, etc.) as asset tags', required=False)

        # Arguments required for web-app discovery and testing
        parser_webapp = subparsers.add_parser ("dast", help = "Discover and test web application using a DAST plugin")
        parser_webapp.add_argument('--url', help='Web application URL', required=True)
        parser_webapp.add_argument('--plugin', choices=['arachni', 'skipfish'], help='DAST plugin to be used. Default is arachni. Requires the plugin to be installed separately.', default='arachni')
        parser_webapp.add_argument('--pluginpath', help='Path where the DAST plugin is installed to be used. Default is /usr/bin.', default='/usr/bin')
        parser_webapp.add_argument('--args', help='Optional extra arguments to be passed to the plugin')
        parser_webapp.add_argument('--assetid', help='A unique ID to be assigned to the discovered webapp asset', required=True)
        parser_webapp.add_argument('--assetname', help='Optional name/label to be assigned to the webapp asset')

        # Arguments required for ssl audit 
        parser_ssl_audit = subparsers.add_parser ("ssl_audit", help = "Run SSL audit tests against your web URLs")
        parser_ssl_audit.add_argument('--url', help='HTTPS URL', required=True)
        parser_ssl_audit.add_argument('--args', help='Optional extra arguments')
        parser_ssl_audit.add_argument('--info', help='Report LOW / INFO level issues', action='store_true')
        parser_ssl_audit.add_argument('--assetid', help='A unique ID to be assigned to the discovered web URL asset', required=True)
        parser_ssl_audit.add_argument('--assetname', help='Optional name/label to be assigned to the web URL asset')

        # Arguments required for AWS CIS benchmarks
        parser_aws_cis = subparsers.add_parser ("aws_cis", help = "Run AWS CIS benchmarks")
        parser_aws_cis.add_argument('--aws_access_key', help='AWS access key', required=True)
        parser_aws_cis.add_argument('--aws_secret_key', help='AWS secret key', required=True)
        parser_aws_cis.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset', required=True)
        parser_aws_cis.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_aws_cis.add_argument('--prowler_home', help='Location of cloned prowler github repo. Defaults to /usr/share/prowler', default='/usr/share/prowler')

        # Arguments required for AWS Audit Checks 
        parser_aws_audit = subparsers.add_parser ("aws_audit", help = "Run AWS audit checks including PCI, GDPR, HIPAA")
        parser_aws_audit.add_argument('--aws_access_key', help='AWS access key', required=True)
        parser_aws_audit.add_argument('--aws_secret_key', help='AWS secret key', required=True)
        parser_aws_audit.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset', required=True)
        parser_aws_audit.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_aws_audit.add_argument('--prowler_home', help='Location of cloned prowler github repo. Defaults to /usr/share/prowler', default='/usr/share/prowler')


        # Arguments required for Azure CIS benchmarks
        parser_az_cis = subparsers.add_parser("azure_cis", help = "Run Azure CIS benchmarks")
        parser_az_cis.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset', required=True)
        parser_az_cis.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')


        # Arguments required for GCP CIS benchmarks
        parser_gcp_cis = subparsers.add_parser("gcp_cis", help = "Run Google Cloud Platform CIS benchmarks")
        parser_gcp_cis.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset', required=True)
        parser_gcp_cis.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_gcp_cis.add_argument('--projects', help='A comma separated list of GCP project IDs to run the checks against')
        parser_gcp_cis.add_argument('--expanded', action='store_true', help='Create separate issue for each violation')
        parser_gcp_cis.add_argument('--custom_ratings', help='Specify JSON file which provides custom ratings for GCP CIS benchmark tests')

        # Arguments required for docker CIS benchmarks 
        parser_docker_cis = subparsers.add_parser ("docker_cis", help = "Run docker CIS benchmarks")
        parser_docker_cis.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset')
        parser_docker_cis.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_docker_cis.add_argument('--docker_bench_home', help='Location of docker bench CLI. Defaults to /usr/share/docker-bench-security', default='/usr/share/docker-bench-security')

        # Arguments required for K8S CIS benchmarks 
        parser_k8s_cis = subparsers.add_parser ("k8s_cis", help = "Run Kubernetes CIS benchmarks")
        parser_k8s_cis.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset', required=True)
        parser_k8s_cis.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_k8s_cis.add_argument('--target', help='Run test against Kubernetes master or worker nodes', choices=['master','worker'], required=True)
        parser_k8s_cis.add_argument('--custom_ratings', help='Specify JSON file which provides custom ratings for Kubernetes CIS benchmarks')

        # Arguments required for GKE CIS benchmarks 
        parser_gke_cis = subparsers.add_parser ("gke_cis", help = "Run GKE CIS benchmarks")
        parser_gke_cis.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset', required=True)
        parser_gke_cis.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_gke_cis.add_argument('--target', help='Run test against GKE master or worker nodes', choices=['master','worker'], required=True)
        parser_gke_cis.add_argument('--custom_ratings', help='Specify JSON file which provides custom ratings for Kubernetes CIS benchmarks')



        args = parser.parse_args()

        logging_level = logging.WARNING
        if args.verbosity >= 1:
            logging_level = logging.INFO
        if args.verbosity >= 2:
            logging_level = logging.DEBUG
        if args.quiet:
            logging_level = logging.ERROR
        # Setup the logger
        logging.basicConfig(filename=logfilename, level=logging_level, filemode='w', format='%(asctime)s %(levelname)-8s %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')
        console = logging.StreamHandler()
        console.setLevel(logging_level)
        console.setFormatter(logging.Formatter('%(levelname)-8s %(message)s'))
        logging.getLogger('').addHandler(console)

        # In insecure mode, we want to set verify=False for requests
        if args.insecure:
            utils.set_requests_verify(False)

        logging.info('Started new run')
        logging.debug('Arguments: %s', str(args))

        if args.mode == "login":
            login_user(args)
        elif args.mode == "logout":
            logout_user(args)

        logged_in_user_dict = { }

        if args.handle is None:
            logged_in_user_dict = get_logged_in_user_details()
            temp = logged_in_user_dict.get('handle')
            if temp is None:
                temp = os.environ.get('TW_HANDLE')
                if temp is None:
                    logging.error('Error: Missing "--handle" argument, user not logged in twigs and "TW_HANDLE" environment variable is not set as well')
                    sys.exit(1)
                logging.info('Using handle specified in "TW_HANDLE" environment variable')
            else:
                logging.info('Using handle of logged in user')
            args.handle = temp

        if args.token is None:
            temp = logged_in_user_dict.get('token')
            if temp is None:
                temp = os.environ.get('TW_TOKEN')
                if temp is not None:
                    logging.info('Using token specified in "TW_TOKEN" environment variable')
            else:
                logging.info('Using token of logged in user')
            args.token = temp

        if args.token is None and args.apply_policy is not None:
            logging.error('Error: Policy cannot be applied since "--token" argument is missing and "TW_TOKEN" environment variable is not set as well!')
            sys.exit(1)

        if args.instance is None:
            temp = logged_in_user_dict.get('instance')
            if temp is None:
                temp = os.environ.get('TW_INSTANCE')
                if temp is not None:
                    logging.info('Using instance specified in "TW_INSTANCE" environment variable')
                    args.instance = temp
                elif args.token is not None:
                    # missing instance but token is specified
                    logging.error('Error: Missing "--instance" argument and "TW_INSTANCE" environment variable is not set as well')
                    sys.exit(1)
            else:
                logging.info('Using instance of logged in user')
                args.instance = temp

#    if args.purge_assets == True and args.email_report == False:
#        logging.error('Purge assets option (--purge_assets) is used with Email report (--email_report)')
#        sys.exit(1)

        if (args.token is None or len(args.token) == 0) and args.out is None:
            logging.error('[token] argument is not specified and [out] argument is not specified. Unable to share discovered assets.')
            sys.exit(1)

        if args.schedule is not None and sys.platform != 'win32':
            from crontab import CronSlices
            # validate schedule
            if CronSlices.is_valid(args.schedule) == False:
                logging.error("Error: Invalid cron schedule [%s] specified!" % args.schedule)
                sys.exit(1)

        assets = []
        sub_pkg_list = ['ssl_audit']
        if args.mode in sub_pkg_list:
            assets = sub_pkg_get_inventory(args)
        elif args.mode == 'aws':
            assets = aws.get_inventory(args)
        elif args.mode == 'azure':
            assets = azure.get_inventory(args)
        elif args.mode == 'acr':
            assets = acr.get_inventory(args)
        elif args.mode == 'ecr':
            assets = ecr.get_inventory(args)
        elif args.mode == 'gcp':
            assets = gcp.get_inventory(args)
        elif args.mode == 'gcr':
            assets = gcr.get_inventory(args)
        elif args.mode == 'servicenow':
            assets = servicenow.get_inventory(args)
        elif args.mode == 'repo':
            assets = repo.get_inventory(args)
        elif args.mode == 'host':
            assets = linux.get_inventory(args)
        elif args.mode == 'vmware':
            assets = vmware.get_inventory(args)
        elif args.mode == 'nmap':
            assets = fingerprint.get_inventory(args)
        elif args.mode == 'docker':
            assets = docker.get_inventory(args)
        elif args.mode == 'k8s':
            assets = kubernetes.get_inventory(args)
        elif args.mode == 'sbom':
            assets = sbom.get_inventory(args)
        elif args.mode == 'file':
            assets = inv_file.get_inventory(args)
        elif args.mode == 'dast':
            assets = dast.get_inventory(args)
        elif args.mode == 'docker_cis':
            assets = docker_cis.get_inventory(args)
        elif args.mode == 'aws_cis':
            assets = aws_cis.get_inventory(args)
        elif args.mode == 'aws_audit':
            assets = aws_cis.get_inventory(args, True)
        elif args.mode == 'azure_cis':
            assets = azure_cis.get_inventory(args)
        elif args.mode == 'gcp_cis':
            assets = gcp_cis.get_inventory(args)
        elif args.mode == 'k8s_cis':
            assets = k8s_cis.get_inventory(args, 'k8s')
        elif args.mode == 'gke_cis':
            assets = k8s_cis.get_inventory(args, 'gke')
        elif args.mode == 'azure_functions':
            assets = azure_functions.get_inventory(args)
        elif args.mode == 'gcloud_functions':
            assets = gcloud_functions.get_inventory(args)
        elif args.mode == 'ssl_audit':
            assets = ssl_audit.get_inventory(args)

        exit_code = None
        if args.mode != 'host' or args.secure == False:
            if assets is None or len(assets) == 0:
                logging.info("No assets found!")
            else:
                """
                if args.asset_criticality is not None:
                    add_asset_criticiality_tag(assets, args.asset_criticality)
                """

                if args.no_auto_tags:
                    remove_standard_tags(assets)

                if args.tag_critical:
                    add_asset_criticality_tag(assets, '5')

                if args.tag:
                    add_asset_tags(assets, args.tag)

                if args.owner:
                    add_asset_owners(assets, args.owner)

                if args.location:
                    add_asset_location(assets, args.location)

                if args.out is not None:
                    export_assets_to_file(assets, args.out)

                if args.token is not None and len(args.token) > 0:
                    asset_id_list, scan_asset_id_list = push_assets_to_TW(assets, args)

                pj_json = None
                if args.apply_policy is not None:
                    policy_job_name = policy_lib.apply_policy(args.apply_policy, asset_id_list, args)
                    while True:
                        time.sleep(60)
                        status, pj_json = policy_lib.is_policy_job_done(policy_job_name, args)
                        if status:
                            exit_code = policy_lib.process_policy_job_actions(pj_json)
                            break

                if args.token is not None and len(args.token) > 0:
                    run_scan(scan_asset_id_list, pj_json, args)
            
                if args.schedule is not None and sys.platform != 'win32':
                    from crontab import CronTab
                    # create/update cron job
                    cron_cmd = sys.argv[0]
                    if "--handle" not in sys.argv:
                        cron_cmd = cron_cmd + " " + "--handle " + '"' + args.handle + '"'
                    if "--token" not in sys.argv:
                        cron_cmd = cron_cmd + " " + "--token " + '"' + args.token + '"'
                    if "--instance" not in sys.argv:
                        cron_cmd = cron_cmd + " " + "--instance " + '"' + args.instance + '"'
                    skip_next = False
                    for index in range(1, len(sys.argv)):
                        if sys.argv[index] == "--schedule":
                            skip_next = True
                            continue
                        if skip_next:
                            skip_next = False
                            continue
                        if sys.argv[index].startswith('-'):
                            cron_cmd = cron_cmd + " " + sys.argv[index]
                        else:
                            cron_cmd = cron_cmd + " " + '"' + sys.argv[index] + '"'
                    cron_comment = "TWIGS_" + args.mode
                    with CronTab(user=True) as user_cron:
                        # Find any existing jobs and remove those
                        ejobs = user_cron.find_comment(cron_comment)
                        for ejob in ejobs:
                            user_cron.remove(ejob)
                        njob = user_cron.new(command=cron_cmd, comment=cron_comment)
                        njob.setall(args.schedule)
                        logging.info("Added to crontab with comment [%s]", cron_comment)

        logging.info('Run completed')
        if exit_code is not None:
            logging.info("Exiting with code [%s] based on policy evaluation", exit_code)
            sys.exit(exit_code)

    except Exception as e:
        logging.error("Something went wrong with this twigs run")
        st = traceback.format_exc()
        logging.error("Exception trace details: %s", st)
        sys.exit(1)

if __name__ == '__main__':
    main()



import sys
import os
import subprocess
import logging
import re

from . import utils

def run_docker_bench(args):
    DBENCH = "/docker-bench-security.sh"

    asset_id = utils.get_ip() if args.assetid is None or args.assetid.strip() == "" else args.assetid
    asset_name = asset_id if args.assetname is None or args.assetname.strip() == "" else args.assetname
    os_release = utils.get_os_release(args, None)
    if os_release == None:
        logging.error('Unsupported OS type for running docker bench')
        sys.exit(1)
    atype = utils.get_asset_type(os_release)

    dbench_path = args.docker_bench_home + DBENCH
    if not os.path.isfile(dbench_path) or not os.access(dbench_path, os.X_OK):
        logging.error('Docker bench script not found')
        sys.exit(1)
    logging.info('Running docker bench script: '+dbench_path)
    cwd = os.getcwd()
    try:
        os.chdir(os.path.dirname(dbench_path))
        out = subprocess.check_output([dbench_path+" 2>/dev/null "], shell=True)
        out = out.decode(args.encoding)
        ansi_escape = re.compile(r'(?:\x1B[@-_]|[\x80-\x9F])[0-?]*[ -/]*[@-~]')
        out = ansi_escape.sub('', out)
    except subprocess.CalledProcessError:
        logging.error("Error running docker bench script")
        return None
    logging.info("docker bench run completed")
    os.chdir(cwd)

    asset_data = {}
    asset_data['id'] = asset_id
    asset_data['name'] = asset_name
    asset_data['type'] = atype
    asset_data['owner'] = args.handle
    asset_data['products'] = []
    asset_tags = []
    asset_tags.append('OS_RELEASE:' + os_release)
    asset_tags.append('Docker')
    asset_tags.append('CIS')
    asset_tags.append('Container')
    asset_tags.append('Linux')
    asset_tags.append(atype)
    asset_data['tags'] = asset_tags

    findings = []
    details = ''
    issue = {}
    for l in out.splitlines():
        if not l.startswith('[WARN]') and not l.startswith('[NOTE]') and not l.startswith('[INFO]'):
            continue
        if l.startswith('[INFO] Checks') or l.startswith('[INFO] Score'):
            continue
        spa = l.split()
        if spa[1] != '*':
            if 'asset_id' in issue:
                if issue['rating'] == '2' and details.strip() == '':
                    details = ''
                    issue = {}
                    continue
                issue['details'] = details
                findings.append(issue)
                details = ''
                issue = {}
            check_id = spa[1].strip()
            issue['twc_id'] = 'docker-bench-check-'+check_id
            issue['asset_id'] = asset_id
            issue['twc_title'] = check_id + ' ' + l.split(check_id)[1].strip()
            if l.startswith('[WARN]'):
                rating = '4'
            elif l.startswith('[NOTE]'):
                rating = '3'
            else:
                rating = '2'
            issue['rating'] = rating 
            issue['type'] = 'Docker CIS'
            issue['object_id'] = ''
            issue['object_meta'] = ''
            details = ''
        else:
            details = details + l.split('*')[1].strip() + '\n'
    # add the final issue
    if 'asset_id' in issue:
        issue['details'] = details
        findings.append(issue)
    asset_data['config_issues'] = findings
    # disable scan
    args.no_scan = True
    return [ asset_data ]

def get_inventory(args):
    return run_docker_bench(args)



import os
import json
import logging
import requests
import sys
from . import az_cis_utils as az_cis_utils
	
def check41(subid):
    logging.info("Processing 41...")
    return ["SQL Server Checks not implemented"]
'''
    try:
        query41=('az account get-access-token --subscription %s --query [accessToken]' % subid)
        json_cis20=az_cis_utils.query_az(query41)
        access_token=json_cis20[0]
        headers = {"Authorization": 'Bearer ' + access_token}
        request = ('https://management.azure.com/subscriptions/%s/providers/Microsoft.Sql/servers?api-version=2015-05-01-preview' % subid)
        try:
            json_output = requests.get(request, headers=headers).json()
            # Query through Azure Server
            for i in range(len(json_output)):
                id = json_output['value'][0][i]['id']
                print(id)
        except Exception as e:
            logging.error('Failing ' + str(e))
            return ["Failed to Query SQL Servers"]
    except Exception as e:
            logging.error('Failing' + str(e))
            return ["Failed to Get Access Token"]

    try:
        json_cis41=os.popen('Get-AzureRmSqlServer | ConvertTo-Json').read()
        #iteration through sql servers
        for i in range(len(json_cis41)):
            RG=json_cis41[i]['ResourceGroupName']
            SRV=json_cis41[i]['ServerName']
            query411=('Get-AzureRmSqlServerAuditing -ResourceGroupName %s -ServerName %s | ConvertTo-Json' % (RG,SRV))
            print(query411)
            json_cis411=os.popen(query411).read()
            print(json_cis411)
    except Exception as e:
        logging.error('Failing ' + str(e))
        return "Failed to Query SQL Servers"
'''	

def check42(subid):
    logging.info("Processing 42...")
    chk421=""
    chk422=""
    chk423=""
    chk424=""
    chk425=""
    chk426=""
    chk427=""
    chk428=""
    totalDBI42 = 0 #Default Master
    passvalue421= 0
    score421=""
    passed421='<font color="green">Passed </font>'
    passvalue422= 0
    score422=""
    passed422='<font color="green">Passed </font>'
    passvalue423= 0
    score423=""
    passed423='<font color="green">Passed </font>'
    passvalue424= 0
    score424=""
    passed424='<font color="green">Passed </font>'
    passvalue425= 0
    score425=""
    passed425='<font color="green">Passed </font>'
    passvalue426= 0
    score426=""
    passed426='<font color="green">Passed </font>'
    passvalue427= 0
    score427=""
    passed427='<font color="green">Passed </font>'
    passvalue428= 0
    score428=""
    passed428='<font color="red">Failed </font>'
    try:
        query42='az sql server list --query "[*].[resourceGroup,name]"'
        json_cis=az_cis_utils.query_az(query42)
        if (len(json_cis)!=0):
            #iteration through sql servers
            for i in range(len(json_cis)):         
                RG=str(json_cis[i][0])
                SRV=str(json_cis[i][1])
                logging.info("Checking server %s within Resource Group %s", SRV,RG)
                try: 
                    query421=("az sql db list --resource-group %s --server %s --query  [*].[name]" % (RG,SRV))
                    json_cis2=az_cis_utils.query_az(query421)
                    #iteration through db instances
                    for j in range(len(json_cis2)):
                        DB=str(json_cis2[j])
                        DB = DB.replace('[\'','')
                        DB = DB.replace('\']','')
                        if DB != "master":
                            query422=('az sql db audit-policy show --resource-group %s --server %s --name %s --query [retentionDays,state]' % (RG,SRV,DB))
                            query423=('az sql db threat-policy show --resource-group %s --server %s --name %s --query [retentionDays,state,disabledAlerts,emailAddresses,emailAccountAdmins]' % (RG,SRV,DB))
                            query424=('az sql db tde show --resource-group %s --server %s --database %s --query [status]' % (RG,SRV,DB))
                            json_cis3=az_cis_utils.query_az(query422)
                            json_cis4=az_cis_utils.query_az(query423)
                            json_cis5=az_cis_utils.query_az(query424)
                            chk421=chk421+('Instance <b>%s</b> on server %s in resource group %s state: <font color="blue"><b>%s</b></font><br>\n' % (DB,SRV,RG,json_cis3[1]))
                            if (json_cis3[1]=="Enabled"):
                                passed421='<font color="green">Passed </font>'
                                passvalue421=passvalue421+1
                            else:
                                passed421='<font color="red">Failed </font>'
                            chk422=chk422+('Instance <b>%s</b> on server %s in resource group %s state: <font color="blue"><b>%s</b></font><br>\n' % (DB,SRV,RG,json_cis4[1]))
                            if (json_cis4[1]=="Enabled"):
                                passed422=='<font color="green">Passed  </font>'
                                passvalue422=passvalue422+1
                            else:
                                passed422='<font color="red">Failed </font>'                        
                            chk423=chk423+('Instance <b>%s</b> on server %s in resource group %s disabledAlerts: <font color="blue"><b>%s</b></font><br>\n' % (DB,SRV,RG,json_cis4[2]))
                            if (json_cis4[2]!=""):
                                passed423=='<font color="green">Passed  </font>'
                                passvalue423=passvalue423+1
                            else:
                                passed423='<font color="red">Failed </font>'                          
                            chk424=chk424+('Instance <b>%s</b> on server %s in resource group %s emailAdd: <font color="blue"><b>%s</b></font><br>\n' % (DB,SRV,RG,json_cis4[3]))
                            if (json_cis4[3]!=""):
                                passed424=='<font color="green">Passed  </font>'
                                passvalue424=passvalue424+1
                            else:
                                passed424='<font color="red">Failed </font>'  
                            chk425=chk425+('Instance <b>%s</b> on server %s in resource group %s emailAdmins: <font color="blue"><b>%s</b></font><br>\n' % (DB,SRV,RG,json_cis4[4]))
                            if (json_cis4[4]=="Enabled"):
                                passed425=='<font color="green">Passed  </font>'
                                passvalue425=passvalue425+1
                            else:
                                passed425='<font color="red">Failed </font>'  
                            chk426=chk426+('Instance <b>%s</b> on server %s in resource group %s tde: <font color="blue"><b>%s</b></font><br>\n' % (DB,SRV,RG,json_cis5[0]))
                            if (json_cis5[0]=="Enabled"):
                                passed426=='<font color="green">Passed  </font>'
                                passvalue426=passvalue426+1
                            else:
                                passed426=='<font color="red">Failed  </font>'
                            chk427=chk427+('Instance <b>%s</b> on server %s in resource group %s retentionDays: <font color="blue"><b>%d</b></font><br>\n' % (DB,SRV,RG,json_cis3[0]))
                            if (json_cis3[0]>=90):
                                passed427=='<font color="green">Passed  </font>'
                                passvalue427=passvalue427+1
                            else:
                                passed427='<font color="red">Failed </font>'  
                            chk428=chk428+('Instance <b>%s</b> on server %s in resource group %s retentionDays: <font color="blue"><b>%d</b></font><br>\n' % (DB,SRV,RG,json_cis4[0]))
                            if (json_cis4[0]>=90):
                                passed428=='<font color="green">Passed  </font>'
                                passvalue428=passvalue428+1
                            else:
                                passed428='<font color="red">Failed </font>'  
                            totalDBI42 = totalDBI42+1
                except Exception as e:
                    #failed="Failed to Query DB"
                    logging.error("Exception in check42: %s %s" %(type(e), str(e.args)))
                    unkScore='<font color="orange">UNKNOWN </font>'
                    chk="Failed to Query DB"
                    uscore=[chk,passvalue421,totalDBI42,unkScore]
                    return [uscore,uscore,uscore,uscore,uscore,uscore,uscore,uscore]
                    #return ["Failed to Query DB"]
            #Minus 1 to remove Master DB which comes by default
            #totalDBI42 = totalDBI42 -1
            score421=[chk421,passvalue421,totalDBI42,passed421]
            score422=[chk422,passvalue422,totalDBI42,passed422]
            score423=[chk423,passvalue423,totalDBI42,passed423]
            score424=[chk424,passvalue424,totalDBI42,passed424]
            score425=[chk425,passvalue425,totalDBI42,passed425]
            score426=[chk426,passvalue426,totalDBI42,passed426]
            score427=[chk427,passvalue427,totalDBI42,passed427]
            score428=[chk428,passvalue428,totalDBI42,passed428]
            return [score421,score422,score423,score424,score425,score426,score427,score428]
        else:
            unkScore='<font color="green">Passed </font>'
            chk="No SQL Servers Found"
            totalDBI42 = 1
            passvalue421 = 1
            uscore=[chk,passvalue421,totalDBI42,unkScore]
            return [uscore,uscore,uscore,uscore,uscore,uscore,uscore,uscore]
            #return ["N/A No SQL Servers Found"]
    except Exception as e:
        logging.error("Exception in check42: %s %s" %(type(e), str(e.args)))
        unkScore='<font color="orange">UNKNOWN</font> '
        chk="Failed to Query Server"
        totalDBI42 = 1
        uscore=[chk,passvalue421,totalDBI42,unkScore]
        return [uscore,uscore,uscore,uscore,uscore,uscore,uscore,uscore]
        #return ["Failed to Query Server"]



import sys
import os
import subprocess
import logging
import tempfile
import shutil
import stat
import tarfile
import re
import traceback
import json
import pkg_resources
import importlib
import io

from . import utils
from . import repo
from . import plugin_processor

docker_cli = ""

def make_temp_directory(tmp_dir):
    if tmp_dir is None:
        temp_dir = tempfile.mkdtemp()
    else:
        temp_dir = tempfile.mkdtemp(dir=tmp_dir)
    return temp_dir

def on_rm_error( func, path, exc_info):
    os.chmod( path, stat.S_IWRITE )
    os.unlink( path )

# Note this routine only handles direct symbolic links and not nested ones
def fix_symbolic_links_helper(container_fs):
    for root, subdirs, files in os.walk(container_fs):
        for fname in files:
            file_path = os.path.join(root, fname)
            if os.path.islink(file_path):
                symlink_path = os.readlink(file_path)
                # Typically absolute links should point within container_fs
                if os.path.isabs(symlink_path) and not symlink_path.startswith(container_fs):
                    nested_path = container_fs + symlink_path
                    if os.path.exists(nested_path):
                        # Fix symbolic link to point within container_fs
                        os.remove(file_path)
                        os.symlink(nested_path, file_path)

def fix_symbolic_links(container_fs):
    # This is a two phase operation as there could be symbolic links that could not be processed
    # due to a symbolic that will be processed later...
    fix_symbolic_links_helper(container_fs)
    fix_symbolic_links_helper(container_fs)

def tar_available():
    if os.path.isfile("/bin/tar"):
        return "/bin/tar"
    elif os.path.isfile("/usr/bin/tar"):
        return "/usr/bin/tar"
    return None

def untar(tar_file, untar_directory):
    tar_cmd = tar_available()
    if tar_cmd is not None:
        tar_failed = False
        cmdarr = [tar_cmd + ' -C ' + untar_directory + ' -xvf ' + tar_file]
        out = ''
        try:
            out = subprocess.check_output(cmdarr, shell=True)
            return
        except subprocess.CalledProcessError:
            logging.error("Unable to untar container image tar file")
            tar_failed = True

    if tar_cmd is None or tar_failed:
        with tarfile.open(tar_file, 'r', format=tarfile.PAX_FORMAT) as tf:
            tf.extractall(path=untar_directory)

def get_container_fs(container_tar):
    if container_tar is None:
        return None
    working_dir = os.path.dirname(container_tar)
    container_dir = working_dir + os.path.sep + 'container'
    os.mkdir(container_dir)
    untar(container_tar, container_dir)
    container_fs = working_dir + os.path.sep + 'container_fs'
    os.mkdir(container_fs)
    layers = []
    with open(container_dir + os.path.sep + 'manifest.json', 'r') as fd:
        manifest_json = json.load(fd)
        layers = manifest_json[0]['Layers']
    for layer in layers:
        layer_tar = container_dir + os.path.sep + layer
        untar(layer_tar, container_fs)
    os.remove(container_tar)
    shutil.rmtree(container_dir, onerror = on_rm_error)
    return container_fs

def unpack_container_fs(container_tar):
    if container_tar is None:
        return None
    working_dir = os.path.dirname(container_tar)
    container_fs = working_dir + os.path.sep + 'container_fs'
    os.mkdir(container_fs)
    untar(container_tar, container_fs)
    os.remove(container_tar)
    return container_fs

def docker_available():
    if os.path.isfile("/usr/bin/docker"):
        return "/usr/bin/docker"
    elif os.path.isfile("/usr/local/bin/docker"):
        return "/usr/local/bin/docker"
    return None 

def save_image(args, working_dir):
    image = args.image
    container_tar = working_dir + os.path.sep + 'container.tar'
    cmdarr = [docker_cli + ' save -o ' + container_tar + ' ' + image]
    out = ''
    try:
        out = subprocess.check_output(cmdarr, shell=True)
        out = out.decode(args.encoding)
    except subprocess.CalledProcessError:
        logging.error("Unable to save container image as tar archive")
        return None
    return container_tar

def export_container(args, working_dir):
    containerid = args.containerid
    container_tar = working_dir + os.path.sep + 'container.tar'
    cmdarr = [docker_cli + ' export -o ' + container_tar + ' ' + containerid]
    out = ''
    try:
        out = subprocess.check_output(cmdarr, shell=True)
        out = out.decode(args.encoding)
    except subprocess.CalledProcessError:
        logging.error("Unable to export container file system as tar archive")
        return None
    return container_tar

def start_docker_container(args):
    if args.image is None:
        return args.containerid
    cmdarr = [docker_cli+' run -d --rm -i -t '+args.image + ' /bin/sh']
    out = ''
    try:
        out = subprocess.check_output(cmdarr, shell=True)
        out = out.decode(args.encoding)
    except subprocess.CalledProcessError:
        logging.error("Unable to start container: "+args.image)
        return None
    container_id = out[:12]
    logging.info("Started container with ID ["+container_id+"] from image ["+args.image+"] for discovery")
    return container_id

def stop_docker_container(args, container_id):
    if args.image is None:
        return
    cmdarr = [docker_cli+' stop '+container_id]
    out = ''
    try:
        out = subprocess.check_output(cmdarr, shell=True)
    except subprocess.CalledProcessError:
        logging.error("Error stopping docker container with container ID ["+container_id+"]")
        return
    logging.info("Stopped container with ID ["+container_id+"]")

def remove_image(args):
    cmdarr = [docker_cli, "image", "rm", "-f", args.image]
    try:
        out = subprocess.check_output(cmdarr)
    except subprocess.CalledProcessError:
        logging.error("Error removing docker image: "+args.image)
        return False
    return True

def pull_image(args):
    cmdarr = [docker_cli, "pull", args.image]
    try:
        out = subprocess.check_output(cmdarr)
    except subprocess.CalledProcessError:
        logging.error("Error pulling docker image: "+args.image)
        return False
    return True

def get_image_id(args):
    cmdarr = [docker_cli, "images", args.image]
    out = ''
    try:
        out = subprocess.check_output(cmdarr)
        out = out.decode(args.encoding)
    except subprocess.CalledProcessError:
        logging.error("Error getting image details: "+args.image)
        return None 
    imageid = None
    for l in out.splitlines():
        if 'REPOSITORY' in l:
            continue
        imageid = l.split()[2]
        break
    return imageid

def get_image_digest(args):
    # During kubernetes discovery, we get container image names as "docker.io/bitnami/wordpress"
    # and we need to strip out the "docker.io" part
    image_tokens = args.image.split('/')
    temp_image = "/".join(image_tokens[-2:]) if len(image_tokens) > 2 else args.image 

    image_tokens = temp_image.split(':')
    image = image_tokens[0]
    image_tag = "latest" if len(image_tokens) == 1 else image_tokens[1]

    cmdarr = [docker_cli, "images", image, "--digests"]
    out = ''
    try:
        out = subprocess.check_output(cmdarr)
        out = out.decode(args.encoding)
    except subprocess.CalledProcessError:
        logging.error("Error getting image digest: "+args.image)
        return None
    image_digest = None
    for l in out.splitlines():
        if 'REPOSITORY' in l:
            continue
        l_tokens = l.split()
        if l_tokens[0] == image and l_tokens[1] == image_tag:
            image_digest = l_tokens[2]
            break
    image_digest = None if image_digest is None or image_digest.strip() == "<none>" else image_digest
    return image_digest

def get_asset_id(args):
    asset_id = None
    if args.assetid is None or args.assetid.strip() == "":
        asset_id = args.image if args.image is not None else args.containerid
    else:
        asset_id = args.assetid
    asset_id = asset_id.replace('/','-')
    asset_id = asset_id.replace(':','-')
    return asset_id

def create_asset(args, os_release, atype, plist, digest, container_fs):
    asset_id = get_asset_id(args)
    asset_name = None
    if args.assetname is None or args.assetname.strip() == "":
        asset_name = args.image if args.image is not None else args.containerid
    else:
        asset_name = args.assetname

    asset_data = {}
    asset_data['id'] = asset_id
    asset_data['name'] = asset_name
    asset_data['type'] = atype
    asset_data['owner'] = args.handle
    asset_data['products'] = plist
    asset_tags = []
    asset_tags.append('OS_RELEASE:' + os_release)
    if args.image is not None:
        asset_tags.append('IMAGE_NAME:' + args.image)
    if args.image is not None and digest is not None:
        asset_tags.append('IMAGE_DIGEST:' + digest)
    asset_tags.append('Docker')
    asset_tags.append('Container')
    asset_tags.append('Linux')
    asset_tags.append(atype)
    asset_data['tags'] = asset_tags

    if container_fs is not None:
        host = {'remote' : False}
        plugin_processor.process_plugins(asset_data, host, args, container_fs)

    return [ asset_data ]

def get_os_release_from_container_image(args, container_fs):
    for root, dirs, files in os.walk(container_fs):
        for f in files:
            tfn = root + os.path.sep + f
            if tfn.endswith('etc/os-release') or tfn.endswith('etc/redhat-release'):
                with open(tfn, 'r') as fd:
                    for line in fd.readlines():
                        if 'PRETTY_NAME' in line:
                            return line.split('=')[1].replace('"','').strip()

    return None

def discover_rh_from_container_image(container_fs):
    # Load rpm module programmatically to avoid display warning message if distro package
    # is not installed in other discovery modes
    try:
        pkg_resources.get_distribution('rpm')
    except pkg_resources.DistributionNotFound as err:
        logging.warning("%s", err)
        logging.warning("python-rpm module not found. Please install [python2-rpm] or [python3-rpm] from your distro package manager")
        return []
    try:
        rpm_module = importlib.import_module('rpm')
    except ImportError as err:
        logging.warning("%s", err)
        return []

    if hasattr(rpm_module, 'addMacro') == False:
            logging.warning('Please install [python2-rpm] or [python3-rpm] from your distro package manager')
            return []
    plist = []
    rpm_addMacro = getattr(rpm_module, 'addMacro')
    rpm_addMacro("_dbpath", container_fs + os.path.sep + os.path.sep.join(["var","lib","rpm"]))
    rpm_TS = getattr(rpm_module, 'TransactionSet')
    ts = rpm_TS()
    packages = ts.dbMatch()
    for ph in packages:
        pkg_name = ph.sprintf("%{NAME} %{VERSION}-%{RELEASE}.%{ARCH}").strip()
        plist.append(pkg_name)
    return plist

def discover_ubuntu_from_container_image(container_fs):
    dpkg_status_file = container_fs + os.path.sep + os.path.sep.join(["var","lib","dpkg","status"])
    plist = []
    pkg = ''
    ver = ''
    with io.open(dpkg_status_file, 'r', errors='ignore') as fd:
        while True:
            line = fd.readline()
            if not line:
                if pkg != '':
                    pkg_ver = pkg + ' ' + ver
                    if pkg_ver not in plist:
                        plist.append(pkg_ver)
                break
            line = line.strip()
            if line == '':
                plist.append(pkg + ' ' + ver)
                pkg = ''
                ver = ''
            if line.startswith('Package:'):
                pkg = line.split(':')[1].strip()
            if line.startswith('Version:'):
                ver = ':'.join(line.split(':')[1:]).strip()
    return plist

def discover_alpine_from_container_image(container_fs):
    apkg_status_file = container_fs + os.path.sep + os.path.sep.join(["lib","apk","db","installed"])
    plist = []
    pkg = ''
    ver = ''
    with io.open(apkg_status_file, 'r', errors='ignore') as fd:
        while True:
            line = fd.readline()
            if not line:
                if pkg != '':
                    pkg_ver = pkg + ' ' + ver
                    if pkg_ver not in plist:
                        plist.append(pkg_ver)
                break
            line = line.strip()
            if line == '':
                plist.append(pkg + ' ' + ver)
                pkg = ''
                ver = ''
            if line.startswith('P:'):
                pkg = line.split(':')[1].strip()
            if line.startswith('V:'):
                ver = line.split(':')[1].strip()
    return plist

def get_opensource_asset_id(args):
    if args.assetid == None:
        args.assetid = args.image
    os_asset_id = repo.get_asset_id(args)
    os_asset_id = os_asset_id + '-opensource'
    return os_asset_id

def create_open_source_asset(args, container_fs, digest):
    args.repo = container_fs
    os_asset_id = get_opensource_asset_id(args)
    oa = repo.discover_inventory(args, container_fs)
    if oa != None and len(oa[0]['products']) != 0:
        oa[0]['name'] = args.assetname+'-container-app'
        oa[0]['id'] = os_asset_id
        oa[0]['type'] = 'Container App'
        oa[0]['tags'].append('IMAGE_NAME:'+args.image)
        if digest is not None:
            oa[0]['tags'].append('IMAGE_DIGEST:'+digest)
    else:
        return None
    return oa

def discover_container_from_image(args, digest):
    casset = None
    temp_dir = None
    try:
        temp_dir = make_temp_directory(args.tmp_dir)
        logging.info("Retrieving container filesystem")
        if args.image is not None:
            container_tar = save_image(args, temp_dir)
            container_fs = get_container_fs(container_tar)
        else:
            container_tar = export_container(args, temp_dir)
            container_fs = unpack_container_fs(container_tar)

        if container_fs is None:
            logging.warning("Unable to analyze container filesystem")
            shutil.rmtree(temp_dir, onerror = on_rm_error)
            return None

        fix_symbolic_links(container_fs)

        oa = create_open_source_asset(args, container_fs, digest)
        if oa != None:
            casset = [] + oa

        logging.info("Analyzing container filesystem")
        os_release = get_os_release_from_container_image(args, container_fs)
        if os_release is None:
            shutil.rmtree(temp_dir, onerror = on_rm_error)
            return casset 

        atype = utils.get_asset_type(os_release)
        if atype is None:
            shutil.rmtree(temp_dir, onerror = on_rm_error)
            return casset 

        plist = None
        if atype == 'CentOS' or atype == 'Red Hat' or atype == 'Amazon Linux' or atype == 'Oracle Linux':
            plist = discover_rh_from_container_image(container_fs)
        elif atype == 'Ubuntu' or atype == 'Debian':
            plist = discover_ubuntu_from_container_image(container_fs)
        elif atype == 'Alpine Linux':
            plist = discover_alpine_from_container_image(container_fs)

        shutil.rmtree(temp_dir, onerror = on_rm_error)
        if plist is None or len(plist) == 0:
            return casset 

        basset = create_asset(args, os_release, atype, plist, digest, container_fs)
        if casset:
            casset = casset + basset 
        else:
            casset = basset

        return casset

    except Exception:
        logging.warning("Unable to discover container from image")
        print(traceback.format_exc())
        if args.containerid is not None:
            logging.error(traceback.format_exc())
        if temp_dir is not None:
            shutil.rmtree(temp_dir, onerror = on_rm_error)
        return None

def get_os_release_from_container_instance(args, container_id):
    base_cmd = docker_cli+' exec -i -t '+container_id+' /bin/sh -c '
    freebsd = False
    out = None
    cmd = '"/bin/cat /etc/os-release"'
    cmdarr = [base_cmd + cmd]
    try:
        out = subprocess.check_output(cmdarr, shell=True)
        out = out.decode(args.encoding)
    except subprocess.CalledProcessError:
        logging.error("Unable to determine os type for container ID: %s", container_id)

    if out is None or out.strip() == '':
        # try FreeBSD
        cmd = '"/usr/bin/uname -v -p"'
        cmdarr = [base_cmd + cmd]
        try:
            out = subprocess.check_output(cmdarr, shell=True)
            out = out.decode(args.encoding)
        except subprocess.CalledProcessError:
            logging.error("Unable to determine os type for container ID: %s", container_id)

        if out is not None and 'FreeBSD' not in out:
            # try OpenBSD
            cmd = '"/usr/bin/uname -srvm"'
            cmdarr = [base_cmd + cmd]
            try:
                out = subprocess.check_output(cmdarr, shell=True)
                out = out.decode(args.encoding)
            except subprocess.CalledProcessError:
                logging.error("Unable to determine os type for container ID: %s", container_id)

    if out is None:
        logging.error("Failed to get os type for container")
        return None

    if 'FreeBSD' in out or 'OpenBSD' in out:
        return out
    else:
        output_lines = out.splitlines()
        for l in output_lines:
            if 'PRETTY_NAME' in l:
                return l.split('=')[1].replace('"','')
    return None

def discover_rh_from_container_instance(args, container_id):
    plist = []
    cmdarr = [docker_cli+' exec -i -t '+container_id+' /bin/sh -c "/usr/bin/yum list installed"']
    yumout = ''
    try:
        yumout = subprocess.check_output(cmdarr, shell=True)
        yumout = yumout.decode(args.encoding)
    except subprocess.CalledProcessError:
        cmdarr = [docker_cli+' exec -i -t '+container_id+' /bin/sh -c "/usr/bin/rpm -qa"']
        rpmout = ''
        try:
            rpmout = subprocess.check_output(cmdarr, shell=True)
            rpmout = rpmout.decode(args.encoding)
        except subprocess.CalledProcessError:
            logging.error("Unable to run inventory for container ID [%s]", container_id)
            return None
        for l in rpmout.splitlines():
            tokens = l.split('-')
            length = len(tokens)
            if length <= 2:
                pname = tokens[0]
                version = tokens[1]
            else:
                version = tokens[length-2]+'-'+tokens[length-1]
                pname = "-".join(tokens[:-2])
            plist.append(pname+' '+version)
        return plist

    begin = False
    for l in yumout.splitlines():
        if 'Installed Packages' in l:
            begin = True
            continue
        if not begin:
            continue
        lsplit = l.split()
        pkg = lsplit[0]
        if len(lsplit) > 1:
            ver = lsplit[1]
        else:
            ver = ''
        pkgsp = pkg.split(".")
        pkg = pkgsp[0]
        arch = pkgsp[1]
        if ':' in ver:
            ver = ver.split(':')[1]
        ver = ver + "." + arch
        pkg_ver = pkg+' '+ver
        pkg_ver = pkg_ver.replace('\x1b[1m','')
        pkg_ver = pkg_ver.replace('\x1b[1','')
        pkg_ver = pkg_ver.replace('\x1b(B','')
        pkg_ver = pkg_ver.replace('\x1b(m','')
        pkg_ver = pkg_ver.replace('\x1b[m','')
        pkg_ver = pkg_ver.replace('\u001b(B','')
        pkg_ver = pkg_ver.replace('\u001b[m','')
        pkg_ver = pkg_ver.replace('\u001b[31m','')
        plist.append(pkg_ver)
    return plist

def discover_ubuntu_from_container_instance(args, container_id):
    plist = []
    cmdarr = [docker_cli+' exec -i -t '+container_id+' /bin/sh -c "/usr/bin/apt list --installed"']
    yumout = ''
    try:
        yumout = subprocess.check_output(cmdarr, shell=True)
        yumout = yumout.decode(args.encoding)
    except subprocess.CalledProcessError:
        logging.error("Unable to run inventory for container ID: "+container_id)
        return None 

    begin = False
    for l in yumout.splitlines():
        if 'Listing...' in l:
            begin = True
            continue
        if not begin:
            continue
        if l.strip() == '':
            continue
        lsplit = l.split()
        pkg = lsplit[0].split('/')[0]
        pkg = pkg.replace('\x1b[32m','')
        pkg = pkg.replace('\x1b[0m','')
        ver = lsplit[1]
        plist.append(pkg+' '+ver)
    return plist

def discover_openbsd_from_container_instance(args, container_id):
    plist = []
    cmdarr = [docker_cli+' exec -i -t '+container_id+' /bin/sh -c "/usr/sbin/pkg_info -A"']
    try:
        pkgout = subprocess.check_output(cmdarr, shell=True)
        pkgout = pkgout.decode(args.encoding)
    except subprocess.CalledProcessError:
        logging.error("Unable to run inventory for container ID: %s",container_id)
        return None

    begin = False
    for l in pkgout.splitlines():
        lsplit = l.split()
        pkgline = lsplit[0]
        ldash = pkgline.rfind('-')
        pkg = pkgline[:ldash] + ' ' + pkgline[ldash + 1:]
        plist.append(pkg)
    return plist

def discover_alpine_from_container_instance(args, container_id):
    plist = []
    cmdarr = [docker_cli+' exec -i -t '+container_id+' /bin/ash -c "/sbin/apk list"']
    try:
        pkgout = subprocess.check_output(cmdarr, shell=True)
        pkgout = pkgout.decode(args.encoding)
    except subprocess.CalledProcessError:
        logging.error("Unable to run inventory for container ID: %s",container_id)
        return None

    begin = False
    for l in pkgout.splitlines():
        if l.startswith('WARNING:'):
            continue
        pkg = l.split()[0]
        ps = pkg.split('-')
        ver = ps[-2] + '-' + ps[-1]
        pkg = pkg.replace('-'+ver, '')
        pkg = pkg + ' ' + ver
        plist.append(pkg)
    return plist

def discover_freebsd_from_container_instance(args, container_id):
    plist = []
    cmdarr = [docker_cli+' exec -i -t '+container_id+' /bin/sh -c "/usr/sbin/pkg info"']
    try:
        pkgout = subprocess.check_output(cmdarr, shell=True)
        pkgout = pkgout.decode(args.encoding)
    except subprocess.CalledProcessError:
        logging.error("Unable to run inventory for container ID: %s",container_id)
        return None

    begin = False
    for l in pkgout.splitlines():
        lsplit = l.split()
        pkgline = lsplit[0]
        ldash = pkgline.rfind('-')
        pkg = pkgline[:ldash] + ' ' + pkgline[ldash + 1:]
        plist.append(pkg)
    return plist

def discover_container_from_instance(args):
    container_id = start_docker_container(args)
    if container_id is None:
        return None

    os_release = get_os_release_from_container_instance(args, container_id)
    if os_release is None:
        stop_docker_container(args, container_id)
        return None
    atype = utils.get_asset_type(os_release)
    if atype is None:
        stop_docker_container(args, container_id)
        return None

    plist = None
    if atype == 'CentOS' or atype == 'Red Hat' or atype == 'Amazon Linux' or atype == 'Oracle Linux':
        plist = discover_rh_from_container_instance(args, container_id)
    elif atype == 'Ubuntu' or atype == 'Debian':
        plist = discover_ubuntu_from_container_instance(args, container_id)
    elif atype == 'FreeBSD':
        plist = discover_freebsd_from_container_instance(args, container_id)
    elif atype == 'OpenBSD':
        plist = discover_openbsd_from_container_instance(args, container_id)
    elif atype == 'Alpine Linux':
        plist = discover_alpine_from_container_instance(args, container_id)

    stop_docker_container(args, container_id)
    if plist == None or len(plist) == 0:
        return None

    return create_asset(args, os_release, atype, plist, None, None)

def run_docker_bench(args):
    DBENCH = "/docker-bench-security.sh"

    asset_id = utils.get_ip() if args.assetid is None or args.assetid.strip() == "" else args.assetid
    asset_name = asset_id if args.assetname is None or args.assetname.strip() == "" else args.assetname
    os_release = utils.get_os_release(args, None)
    if os_release == None:
        logging.error('Unsupported OS type for running docker bench')
        return None
    atype = utils.get_asset_type(os_release)

    dbench_path = args.docker_bench_home + DBENCH
    if not os.path.isfile(dbench_path) or not os.access(dbench_path, os.X_OK):
        logging.error('Docker bench script not found')
        return None
    logging.info('Running docker bench script: '+dbench_path)
    try:
        os.chdir(os.path.dirname(args.docker_bench_home))
        out = subprocess.check_output([dbench_path+" 2>/dev/null "], shell=True)
        out = out.decode(args.encoding)
        ansi_escape = re.compile(r'(?:\x1B[@-_]|[\x80-\x9F])[0-?]*[ -/]*[@-~]')
        out = ansi_escape.sub('', out)
    except subprocess.CalledProcessError:
        logging.error("Unable to run docker bench script")
        return None 

    asset_data = {}
    asset_data['id'] = asset_id
    asset_data['name'] = asset_name
    asset_data['type'] = atype
    asset_data['owner'] = args.handle
    asset_data['products'] = [] 
    asset_tags = []
    asset_tags.append('OS_RELEASE:' + os_release)
    asset_tags.append('Docker')
    asset_tags.append('Container')
    asset_tags.append('Linux')
    asset_tags.append(atype)
    asset_data['tags'] = asset_tags 

    findings = []
    details = ''
    issue = {}
    for l in out.splitlines():
        if not l.startswith('[WARN]'):
            continue
        spa = l.split()
        if spa[1] != '*':
            if 'asset_id' in issue:
                issue['details'] = details
                findings.append(issue)
                details = ''
                issue = {}
            issue['twc_id'] = 'docker-bench-check-'+spa[1].strip()
            issue['asset_id'] = asset_id 
            issue['twc_title'] = l.split('-')[1].strip()
            issue['rating'] = '4'
            issue['object_id'] = '' 
            issue['object_meta'] = ''
            details = ''
        else:
            details = details + l.split('*')[1] + '\n'
    # add the final issue
    if 'asset_id' in issue:
        issue['details'] = details
        findings.append(issue)
    asset_data['config_issues'] = findings
    # disable scan
    args.no_scan = True
    return [ asset_data ]

# digest = None --> get digest using docker
def get_inventory(args, digest=None):


    global docker_cli

    if os.geteuid() != 0:
        logging.error("Docker operations need root privilege. Please run as 'sudo' or 'root'")
        return None

    docker_cli = docker_available()
    if not docker_cli:
        logging.error("Docker CLI not available")
        return None

    if args.image is None and args.containerid is None:
        logging.error("Either docker image (--image) or running container id (--containerid) parameter needs to be specified")
        return None

    if args.tmp_dir is not None and os.path.isdir(args.tmp_dir) == False:
        logging.error("Specified temporary directory [%s] does not exist!", args.tmp_dir)
        return None

    # If digest is not specified, then try to pull image locally to obtain digest
    del_image = False
    if digest is None:
        if args.image is not None:
            if not get_image_id(args):
                if not pull_image(args):
                    logging.error("Failed to pull image: "+args.image)
                    return None
                else:
                    del_image = True
            digest = get_image_digest(args)

    # If image digest is available, then check if image has changed using digest
    if digest is not None and digest != -1:
        if (args.token is None or len(args.token) == 0):
            logging.warn("Unable to compare image digest as [token] argument is not specified")
        else:
            no_change = False
            digest_tag = "IMAGE_DIGEST:%s" % digest
            eos_asset_id = get_opensource_asset_id(args)
            eos_asset = utils.get_asset(eos_asset_id, args)
            if eos_asset is not None:
                for tag in eos_asset['tags']:
                    if digest_tag == tag:
                        no_change = True
            easset_id = get_asset_id(args)
            easset = utils.get_asset(easset_id, args)
            if easset is not None:
                for tag in easset['tags']:
                    if digest_tag == tag:
                        no_change = True
            if no_change:
                logging.info("No change in image digest for container image [%s]...skipping it", args.image)
                if del_image:
                    remove_image(args)
                return None
    digest = None if digest == -1 else digest

    if args.image is not None:
        # Note if digest was passed, then image may not be available locally
        if not get_image_id(args):
            if not pull_image(args):
                logging.error("Failed to pull image: "+args.image)
                return None
            else:
                del_image = True
        assets = discover_container_from_image(args, digest)
        if assets is None and args.start_instance:
            assets = discover_container_from_instance(args)
    elif args.containerid is not None:
        assets = discover_container_from_instance(args)
        if assets is None:
            assets = discover_container_from_image(args, digest)

    # if image was downloaded by twigs then remove it
    if del_image:
        remove_image(args)

    if assets is None:
        logging.error("Unable to inventory container")
        return None

    return assets



import os

# Please use lower case key names i.e. Vulnerability ID's
plugin_registry = {
        "cve-2021-44228": {"name": "Check for CVE-2021-44228 (log4j)", "file": "cve-2021-44228.sh", "enabled": True},
        "cve-2021-45046": {"name": "Check for CVE-2021-45046 (log4j)", "file": "cve-2021-45046.sh", "enabled": True},
        "cve-2021-45105": {"name": "Check for CVE-2021-45105 (log4j)", "file": "cve-2021-45105.sh", "enabled": True},
        "cve-2021-44832": {"name": "Check for CVE-2021-44832 (log4j)", "file": "cve-2021-44832.sh", "enabled": True}
}

def get_plugin_dir():
    return os.path.dirname(os.path.abspath(__file__)) + os.sep + "plugins"

def get_plugin_registry():
    return plugin_registry



import os
import json
import logging
import requests
import sys
from . import az_cis_utils as az_cis_utils

def check11(subid):
    logging.info("Processing 11...")
    
    try:
        query11=('az account get-access-token --subscription %s --query [accessToken]' % subid)
        #score11=['<font color="red">Failed</font>',0]
        json_cis=az_cis_utils.query_az(query11)
        access_token=json_cis[0]
        headers = {"Authorization": 'Bearer ' + access_token}
        request1 = ('https://management.azure.com/subscriptions/%s/providers/Microsoft.Authorization/roleDefinitions?api-version=2017-05-01' % subid)
        request2 = ('https://management.azure.com/subscriptions/%s/providers/Microsoft.Authorization/roleassignments?api-version=2017-10-01-preview' % subid)
        try:
            json_output1 = requests.get(request1, headers=headers).json()
            json_output2 = requests.get(request2, headers=headers).json()
            for j in range(len(json_output2['value'])):
                pType=json_output2['value'][j]['properties']['principalType']
                pid=json_output2['value'][j]['properties']['principalId']
                rid=json_output2['value'][j]['properties']['roleDefinitionId']
                for i in range(len(json_output1['value'])):
                    rolename=json_output1['value'][i]['properties']['roleName']
                    nameB=json_output1['value'][i]['name']
                    if ("Owner" in rolename or "Admin" in rolename  or "Contributor" in rolename and nameB in rid):
                        query111=('az ad user list --query "[?objectId==\'%s\'][userPrincipalName]"' % pid)
                        #json_cis2=az_cis_utils.query_az(query111)
                        #upn=json_cis2[0][0]
                        #print(upn)
                        unkScore=['<font color="orange">UNKNOWN </font>',0]
                        chk="No Supported"
                        return [chk,unkScore]                        
        except Exception as e:
            logging.error("Exception in check11: %s %s" %(type(e), str(e.args)))
            unkScore=['<font color="orange">UNKNOWN </font>',0]
            chk="Failed to make API call"
            return [chk,unkScore]
    except Exception as e:
        logging.error("Exception in check11: %s %s" %(type(e), str(e.args)))
        unkScore=['<font color="orange">UNKNOWN </font>',0]
        chk="Failed to Query"
        return [chk,unkScore]

def check12():
    logging.info("Processing 12...")
    return ["Check not available with azure CLI"]

def check13():
    logging.info("Processing 13...")
    st13=""
    passvalue13 = 0
    totalvalue13 = 0
    score13=""
    passed13='<font color="green">Passed </font>'
    try:
        query13='az ad user list --query "[?userType==\'Guest\']"'
        #query13=""
        json_cis=az_cis_utils.query_az(query13)
        if (len(json_cis)>0):
            #iteration through roles
            passed13='<font color="red">Failed </font>'
            totalvalue13 = len(json_cis)
            st13=("%d Guest users Found" % totalvalue13)
        else:
            st13="No Guest users Found"
            passvalue13 = 1
            totalvalue13 = 1
        
        score13=[st13,passvalue13,totalvalue13,passed13]
        return score13
    except Exception as e:
        logging.error("Exception in check13: %s %s" %(type(e), str(e.args)))
        st13="Failed to query users"
        passed13='<font color="orange">UNKNOWN </font>'
        totalvalue13 = 1
        score13=[st13,passvalue13,totalvalue13,passed13]
        return score13


def check14():
    logging.info("Processing 14...")
    return ["Check not available with azure CLI"]

def check15():
    logging.info("Processing 15...")
    return ["Check not available with azure CLI"]

def check16():
    logging.info("Processing 16...")
    return ["Check not available with azure CLI"]

def check17():
    logging.info("Processing 17...")
    return ["Check not available with azure CLI"]

def check18():
    logging.info("Processing 18...")
    return ["Check not available with azure CLI"]

def check19():
    logging.info("Processing 19...")
    return ["Check not available with azure CLI"]

def check110():
    logging.info("Processing 110...")
    return ["Check not available with azure CLI"]

def check111():
    logging.info("Processing 111...")
    return ["Check not available with azure CLI"]

def check112():
    logging.info("Processing 112...")
    return ["Check not available with azure CLI"]

def check113():
    logging.info("Processing 113...")
    return ["Check not available with azure CLI"]

def check114():
    logging.info("Processing 114...")
    return ["Check not available with azure CLI"]

def check115():
    logging.info("Processing 115...")
    return ["Check not available with azure CLI"]

def check116():
    logging.info("Processing 116...")
    return ["Check not available with azure CLI"]

def check117():
    logging.info("Processing 117...")
    return ["Check not available with azure CLI"]

def check118():
    logging.info("Processing 118...")
    return ["Check not available with azure CLI"]

def check119():
    logging.info("Processing 119...")
    return ["Check not available with azure CLI"]

def check120():
    logging.info("Processing 120...")
    return ["Check not available with azure CLI"]

def check121():
    logging.info("Processing 121...")
    return ["Check not available with azure CLI"]

def check122():
    logging.info("Processing 122...")
    return ["Check not available with azure CLI"]

def check123():
    logging.info("Processing 123...")
    st123=""
    passvalue123 = 0
    failvalue123 = 0
    totalvalue123 = 0
    score123=""
    passed123='<font color="green">Passed </font>'
    try:
        query123='az role definition list --query [*][roleName,assignableScopes,permissions[].actions]'
        json_cis=az_cis_utils.query_az(query123)
        if (len(json_cis)>0):
            #iteration through roles
            for i in range(len(json_cis)):
                role = json_cis[i][0]
                scope= json_cis[i][1][0]
                actions = json_cis[i][2][0]
                #iteration through actions
                if (len(actions)>0):
                    for j in range(len(actions)):
                        if (scope=="/"  and actions[j]=="*"):
                            st123=st123+('Role <b>%s</b> with unrestricted access <br>\n' % role)
                            passed123='<font color="red">Failed </font>'
                            failvalue123=failvalue123+1
                    totalvalue123 = totalvalue123+1
                    passvalue123 = totalvalue123 - failvalue123
                else:
                    st123=st123+('No actions found for role <b>%s</b> with unrestricted access <br>\n' % role)
        else:
            st123="Roles not found"
        score123=[st123,passvalue123,totalvalue123,passed123]
        return score123
    except Exception as e:
        logging.error("Exception in check123: %s %s" %(type(e), str(e.args)))
        st123="Failed to query definition role"
        passed123='<font color="orange">UNKNOWN </font>'
        totalvalue123 = 1
        score123=[st123,passvalue123,totalvalue123,passed123]
        return score123
